C**********************************************************************C
C                                                                      C
C                          J E T N E T - 2.0                           C
C                                                                      C
C           A Neural Network program for jet discrimination            C
C         and other High Energy Physics triggering situations          C
C                                                                      C
C                    Latest date of change 91.09.15                    C
C                                                                      C
C                              Authors :                               C
C                                                                      C
C                  Leif Lonnblad, Carsten Peterson,                    C
C                Hong Pi and Thorsteinn Rognvaldsson                   C
C                                                                      C
C                   Department of Theoretical Physics                  C
C                  University of Lund, Solvegatan 14A,                 C
C                           S-223 62 Lund                              C
C                               Sweden                                 C
C                                                                      C
C                        tel  int+46-46109073                          C
C                        fax  int+46-46104438                          C
C                                                                      C
C                     BITNET/EARN THEPLL@SELDC52                       C
C                                 THEPCAP@SELDC52                      C
C                                 THEPHP@SELDC52                       C
C                                 THEPDR@SELDC52                       C
C                                                                      C
C                     internet    leif@thep.lu.se                      C
C                                 carsten@thep.lu.se                   C
C                                 pihong@thep.lu.se                    C
C                                 denni@thep.lu.se                     C
C                                                                      C
C        Copyright 1991 Leif Lonnblad & Thorsteinn Rognvaldsson        C
C                                                                      C
C                Please report any errors to the authors               C
C                                                                      C
C**********************************************************************C
 
C**********************************************************************C
C  A separate manual is available via bitnet request.                  C
C                                                                      C
C  A description of the model and the program can be found in:         C
C                                                                      C
C  Lonnblad et. al., "Pattern recognition in High Energy Physics with  C
C  Artificial Neural Networks - JETNET 2.0"                            C
C  (Lund University Report LU-TP 91-18)                                C
C                                                                      C
C**********************************************************************C
C
C**********************************************************************C
C                   Order of subroutines and functions:                C
C                                                                      C
C 1) Feed-forward network (JN):                                        C
C         GAUSJN, GJN, GPJN, JNCHOP, JNDELT, JNDUMP, JNERR,            C
C          JNFEED, JNHEAD, JNINDX, JNINIT, JNREAD, JNROLD,             C
C          JNSATM, JNSEFI, JNSEPA, JNSTAT, JNTEST, JNTRAL              C
C                                                                      C
C 2) Self-organizing network (JM):                                     C
C     GJM, JMDUMP, JMERR, JMFEED, JMINDX, JMINIT, JMINWE, JMNBHD,      C
C       JMNORM, JMREAD, JMSEPA, JMSTAT, JMTEST, JMTRAL, JMWARN         C
C                                                                      C
C The block-data subroutine JNDATA is placed at the end, together      C
C with a test-deck in the subroutines JNTDEC and JNTDPT.               C
C**********************************************************************C
C**********************************************************************C
C
C**********************************************************************C
C PART ONE: FEED-FORWARD NETWORK                                       C
C**********************************************************************C
 
 
      REAL FUNCTION GAUSJN(IDUM)
C...JetNet function GAUSsian random number.
 
C...Generates gaussian distributed random numbers with
C...standard deviation 1.0 and mean 0.0. Polar method.
 
      COMMON /JNGAUS/ ISET,GASDEV
      SAVE /JNGAUS/
 
      IF (ISET.EQ.0) THEN
100     V1=2.*RLU(0)-1.
        V2=2.*RLU(0)-1.
        R=V1**2+V2**2
        IF (R.GE.1.) GOTO 100
C...Box-Muller transformation:
        FAC=SQRT(-2.*LOG(R)/R)
        GAUSJN=V1*FAC
        GASDEV=V2*FAC
        ISET=1
      ELSE
        GAUSJN=GASDEV
        ISET=0
      ENDIF
 
      RETURN
 
C**** END OF GAUSJN ****************************************************
      END
C***********************************************************************
 
 
      REAL FUNCTION GJN(X,N)
 
C...JetNet function G
 
C...Gives sigmoid function N with argument x
 
      PARAMETER(MAXI=1000,MAXO=1000)
 
      COMMON /JNDAT1/ MSTJN(40),PARJN(40),MSTJM(20),PARJM(20),
     &                OIN(MAXI),OUT(MAXO),MXNDJM
      SAVE /JNDAT1/
 
 
      IF(N.EQ.1) THEN
        GJN=0.5*(1.0+TANH(X))
      ELSEIF(N.EQ.2) THEN
        GJN=TANH(X)
      ELSEIF(N.EQ.3) THEN
        GJN=EXP(MAX(-50.0,MIN(X,50.0)))
      ELSEIF(N.EQ.4) THEN
        GJN=X
      ELSEIF(N.EQ.5) THEN
        GJN=0.5*(1.0+TANH(X))
      ELSEIF(N.EQ.-1) THEN
        NS=2**ABS(MSTJN(28))
        SS=1.0/(NS-1)
        G=0.5*(1.0+TANH(X))
        NG=INT(G/SS+0.5)
        GJN=FLOAT(NG)*SS
      ELSEIF(N.EQ.-2) THEN
        NS=2**(ABS(MSTJN(28))-1)
        IF(NS.EQ.1) THEN
          GJN=SIGN(1.0,TANH(X))
        ELSE
          SS=1.0/(NS-1)
          G=TANH(X)
          NG=INT(ABS(G)/SS+0.5)
          GJN=SIGN(FLOAT(NG)*SS,G)
        ENDIF
      ELSE
        MSTJN(3)=N
        CALL JNERR(15)
      ENDIF
 
      RETURN
 
C**** END OF GJN *******************************************************
      END
C***********************************************************************
 
 
      REAL FUNCTION GPJN(X,N)
 
C...JetNet function GPrime
 
C...Gives derivative of sigmoid function N with argument x
 
 
      IF(N.EQ.1) THEN
        GPJN=0.5*(1.0-TANH(X)**2)
      ELSEIF(N.EQ.2) THEN
        GPJN=1.0-TANH(X)**2
      ELSEIF(N.EQ.3) THEN
        GPJN=1.0
      ELSEIF(N.EQ.4) THEN
        GPJN=1.0
      ELSEIF(N.EQ.5) THEN
        GPJN=2.0
      ELSE
        GPJN=0.0
      ENDIF
 
      RETURN
 
C**** END OF GPJN ******************************************************
      END
C***********************************************************************
 
 
      SUBROUTINE JNCHOP(ICHP)
C...JetNet subroutine CHOP weights
 
C...Switches on (ICHP>0) or of (ICHP<0) fixed precision weights
C...thresholds and sigmoid functions.For ichp >= 0 the weights and
C...thresholds are chopped to the fixed precision.
 
      PARAMETER(MAXV=2000,MAXM=150000,MAXI=1000,MAXO=1000)
 
      COMMON /JNDAT1/ MSTJN(40),PARJN(40),MSTJM(20),PARJM(20),
     &                OIN(MAXI),OUT(MAXO),MXNDJM
      COMMON /JNINT1/ O(MAXV),A(MAXV),D(MAXV),T(MAXV),DT(MAXV),
     &                W(MAXM),DW(MAXM),NSELF(MAXM),NTSELF(MAXV)
      COMMON /JNINT2/ M(0:10),MV0(10),MM0(10),NG(10),NL,IPOTT,
     &                ER1,ER2,SM(10),ICPON
      SAVE /JNDAT1/,/JNINT1/,/JNINT2/
 
      IF(ICHP.GT.0) THEN
        ICPON=1
        DO 100 I=1,NL-1
          IF(ABS(NG(I)).NE.1.AND.ABS(NG(I)).NE.2) CALL JNERR(14)
          IF(MSTJN(28).NE.0) THEN
            NG(I)=-ABS(NG(I))
          ELSE
            NG(I)=ABS(NG(I))
          ENDIF
100     CONTINUE
        IF(MSTJN(28).GT.0) THEN
          IF(ABS(NG(NL)).NE.1.AND.ABS(NG(NL)).NE.2) CALL JNERR(14)
          NG(NL)=-ABS(NG(NL))
        ELSE
          NG(NL)=ABS(NG(NL))
        ENDIF
 
      ENDIF
 
      IF(ICHP.LT.0) THEN
        ICPON=0
        DO 110 I=1,NL
          NG(I)=ABS(NG(I))
110     CONTINUE
      ENDIF
 
      IF(ICHP.GE.0) THEN
 
        NSW=2**(MSTJN(30)-1)
        NST=2**(MSTJN(29)-1)
 
        DO 200 IL=1,NL
 
          IF(MSTJN(30).GE.1) THEN
            WMAX=0.0
            DO 210 I=MM0(IL)+1,MM0(IL+1)
              WMAX=MAX(WMAX,ABS(W(I)))
210         CONTINUE
            SS=1.0
            IF(NSW.GT.1) THEN
              SS=WMAX/(NSW-1)
            ENDIF
            DO 220 I=MM0(IL)+1,MM0(IL+1)
             IF(NSW.GT.1) THEN
                W(I)=SIGN(FLOAT(INT(ABS(W(I))/SS+0.5))*SS,W(I))
              ELSE
                W(I)=SIGN(WMAX,W(I))
              ENDIF
220         CONTINUE
          ENDIF
 
          IF(MSTJN(29).GE.1) THEN
            TMAX=0.0
            DO 230 I=MV0(IL)+1,MV0(IL+1)
              TMAX=MAX(TMAX,ABS(T(I)))
230         CONTINUE
            SS=1.0
            IF(NST.GT.1) THEN
              SS=TMAX/(NST-1)
            ENDIF
            DO 240 I=MV0(IL)+1,MV0(IL+1)
              IF(NST.GT.1) THEN
                T(I)=SIGN(FLOAT(INT(ABS(T(I))/SS+0.5))*SS,T(I))
              ELSE
                T(I)=SIGN(TMAX,T(I))
              ENDIF
240         CONTINUE
          ENDIF
 
200     CONTINUE
 
      ENDIF
 
      RETURN
 
C**** END OF JNCHOP ****************************************************
      END
C***********************************************************************
 
 
      SUBROUTINE JNDELT
 
C...JetNet subroutine DELTa weights
 
C...Calculates the change in weights and thresholds to minimize the
C...cost function according to gradient descent
 
      PARAMETER(MAXV=2000,MAXM=150000,MAXI=1000,MAXO=1000)
 
      COMMON /JNDAT1/ MSTJN(40),PARJN(40),MSTJM(20),PARJM(20),
     &                OIN(MAXI),OUT(MAXO),MXNDJM
      COMMON /JNDAT2/ TINV(10),IGFN(10),ETAL(10),WIDL(10),SATM(10)
      COMMON /JNINT1/ O(MAXV),A(MAXV),D(MAXV),T(MAXV),DT(MAXV),
     &                W(MAXM),DW(MAXM),NSELF(MAXM),NTSELF(MAXV)
      COMMON /JNINT2/ M(0:10),MV0(10),MM0(10),NG(10),NL,IPOTT,
     &                ER1,ER2,SM(10),ICPON
      COMMON /JNINT3/ NXIN,NYIN,NXRF,NYRF,NXHRF,NYHRF,NHRF,NRFW,NHPRF
      SAVE /JNDAT1/,/JNDAT2/,/JNINT1/,/JNINT2/,/JNINT3/
 
 
C...set beta in last layer
 
      IF(TINV(NL).EQ.0.0) THEN
        BETA=PARJN(3)
      ELSE
        BETA=ABS(TINV(NL))
      ENDIF
 
C...calculate the deltas at nodes in output layer
 
      DO 100 I=1,M(NL)
        MI=MV0(NL)+I
        D(MI)=(OUT(I)-O(MI))*GPJN(BETA*A(MI),ABS(NG(NL)))*BETA
100   CONTINUE
 
C...calculate deltas in following layers
 
      DO 200 IL=NL-1,1,-1
 
C...set beta in layer IL
 
       IF(TINV(IL).EQ.0.0) THEN
         BETA=PARJN(3)
       ELSE
         BETA=ABS(TINV(IL))
       ENDIF
 
C...calculate the deltas at nodes in layer IL
 
       DO 210 J=1,M(IL)
         MJ=MV0(IL)+J
         MIJ=MM0(IL+1)+(J-1)*M(IL+1)
         SUM=0.0
         DO 220 I=MV0(IL+1)+1,MV0(IL+1)+M(IL+1)
           MIJ=MIJ+1
           SUM=SUM+D(I)*W(MIJ)
220      CONTINUE
         D(MJ)=SUM*GPJN(BETA*A(MJ),ABS(NG(IL)))*BETA
210     CONTINUE
200   CONTINUE
 
C...set eta in first layer
 
      IF(ETAL(1).EQ.0.0) THEN
        ETA=PARJN(1)
      ELSE
        ETA=ETAL(1)
      ENDIF
 
C...calculate deltas at all weights between first and second layer
 
      NEXTL=2
 
      IF(NXIN.EQ.0) THEN
 
C...normal first layer
 
        DO 300 I=1,M(1)
          MIJ=I-M(1)
          DO 310 J=1,M(0)
            MIJ=MIJ+M(1)
            DW(MIJ)=DW(MIJ)+ETA*D(I)*OIN(J)
310       CONTINUE
          DT(I)=DT(I)+ETA*D(I)
300     CONTINUE
 
      ELSE
 
C...receptive fields in first layer
 
        DO 320 IHPRF=1,NHPRF
 
          SUMRFT=0.0
 
          DO 330 IY=1,NYHRF
            IH=IY-NYHRF+(IHPRF-1)*NHRF
            DO 340 IX=1,NXHRF
              IH=IH+NYHRF
              DO 350 JY=1,NYRF
                IW=JY-NYRF+(IHPRF-1)*NRFW
                DO 360 JX=1,NXRF
                  IW=IW+NYRF
                  INX=IX+JX-1
                  IF(INX.GT.ABS(NXIN)) INX=INX-ABS(NXIN)
                  INY=IY+JY-1
                  IF(INY.GT.ABS(NYIN)) INY=INY-ABS(NYIN)
                  IN=(INX-1)*ABS(NYIN)+INY
                  DW(IW)=DW(IW)+ETA*D(IH)*OIN(IN)/FLOAT(NHRF)
c                  DW(IW)=DW(IW)+ETA*D(IH)*OIN(IN)
360             CONTINUE
350           CONTINUE
              DO 370 IN=ABS(NXIN*NYIN)+1,M(0)
                IW=NXRF*NYRF+IN-ABS(NXIN*NYIN)+(IHPRF-1)*NRFW
                DW(IW)=DW(IW)+ETA*D(IH)*OIN(IN)/FLOAT(NHRF)
c                DW(IW)=DW(IW)+ETA*D(IH)*OIN(IN)
370           CONTINUE
              SUMRFT=SUMRFT+ETA*D(IH)
340         CONTINUE
330       CONTINUE
 
          SUMRFT=SUMRFT/FLOAT(NXHRF*NYHRF)
 
          DO 380 IH=1,NXHRF*NYHRF
            DT(IH+(IHPRF-1)*NHRF)=DT(IH+(IHPRF-1)*NHRF)+SUMRFT
380       CONTINUE
 
320     CONTINUE
 
        DO 390 IH=NHRF*NHPRF+1,M(1)
          IW=NRFW*NHPRF+IH-M(1)
          DO 400 IN=1,M(0)
            IW=IW+M(1)
            DW(IW)=DW(IW)+ETA*D(IH)*OIN(IN)
400       CONTINUE
          DT(IH)=DT(IH)+ETA*D(IH)
390     CONTINUE
 
        IF(MSTJN(27).LT.0) THEN
 
          IF(ETAL(2).EQ.0.0) THEN
            ETA=PARJN(1)
          ELSE
            ETA=ETAL(2)
          ENDIF
 
          DO 500 I=1,M(2)
            MI=MV0(2)+I
            DO 510 IHPRF=1,NHPRF
              SUMRFW=0.0
              DO 520 J=(IHPRF-1)*NHRF+1,IHPRF*NHRF
                SUMRFW=SUMRFW+ETA*D(MI)*O(J)
520           CONTINUE
              MIJ=MM0(2)+((IHPRF-1)*NHRF-1)*M(2)+I
              SUMRFW=SUMRFW/FLOAT(NHRF)
              DO 530 J=1,NHRF
                MIJ=MIJ+M(2)
                DW(MIJ)=DW(MIJ)+SUMRFW
530           CONTINUE
510         CONTINUE
            MIJ=MM0(2)+I+(NHRF*NHPRF-1)*M(2)
            DO 540 J=NHRF*NHPRF+1,M(1)
              MIJ=MIJ+M(2)
              DW(MIJ)=DW(MIJ)+ETA*D(MI)*O(J)
540         CONTINUE
            DT(MI)=DT(MI)+ETA*D(MI)
500       CONTINUE
 
          NEXTL=3
 
        ENDIF
 
      ENDIF
 
C...calculate deltas at all weights between following layers
 
      DO 410 IL=NEXTL,NL
 
C...set eta in layer IL
 
       IF(ETAL(IL).EQ.0.0) THEN
         ETA=PARJN(1)
       ELSE
         ETA=ETAL(IL)
       ENDIF
 
       DO 420 I=1,M(IL)
         MIJ=MM0(IL)+I-M(IL)
         MI=MV0(IL)+I
         DO 430 J=MV0(IL-1)+1,MV0(IL-1)+M(IL-1)
           MIJ=MIJ+M(IL)
           DW(MIJ)=DW(MIJ)+ETA*D(MI)*O(J)
430      CONTINUE
         DT(MI)=DT(MI)+ETA*D(MI)
420     CONTINUE
410   CONTINUE
 
      RETURN
 
C**** END OF JNDELT ****************************************************
      END
C***********************************************************************
 
 
      SUBROUTINE JNDUMP(NF)
 
C...JetNet subroutine DUMP weights
 
C...Dumps weights, threshold and other characteristics of the
C...net to a file for use in other programs
 
      PARAMETER(MAXV=2000,MAXM=150000,MAXI=1000,MAXO=1000)
 
      COMMON /JNDAT1/ MSTJN(40),PARJN(40),MSTJM(20),PARJM(20),
     &                OIN(MAXI),OUT(MAXO),MXNDJM
      COMMON /JNDAT2/ TINV(10),IGFN(10),ETAL(10),WIDL(10),SATM(10)
      COMMON /JNINT1/ O(MAXV),A(MAXV),D(MAXV),T(MAXV),DT(MAXV),
     &                W(MAXM),DW(MAXM),NSELF(MAXM),NTSELF(MAXV)
      COMMON /JNINT2/ M(0:10),MV0(10),MM0(10),NG(10),NL,IPOTT,
     &                ER1,ER2,SM(10),ICPON
      COMMON /JNINT3/ NXIN,NYIN,NXRF,NYRF,NXHRF,NYHRF,NHRF,NRFW,NHPRF
      SAVE /JNDAT1/,/JNDAT2/,/JNINT1/,/JNINT2/,/JNINT3/
 
      CHARACTER C(0:1)
 
 
      IF(NF.LT.0) THEN
 
C...unformatted dump
 
        JF=-NF
        WRITE(JF) 20
        WRITE(JF) MSTJN,PARJN,TINV,IGFN,ETAL,WIDL,SATM
 
        DO 100 I=1,MM0(NL+1)
          WRITE(JF) W(I)
100     CONTINUE
 
        DO 110 I=1,MV0(NL+1)
          WRITE(JF) T(I)
110     CONTINUE
 
        DO 120 I=1,MM0(NL+1)
          WRITE(JF) NSELF(I)
120     CONTINUE
 
        DO 130 I=1,MV0(NL+1)
          WRITE(JF) NTSELF(I)
130     CONTINUE
 
      ELSE
 
C...Formatted dump
 
        C(1)=' '
        C(0)='*'
 
        NFSAVE=MSTJN(6)
        MSTJN(6)=NF
 
        WRITE(NF,600)
        CALL JNHEAD
        CALL JNSTAT(1)
        CALL JNSTAT(2)
 
        MSTJN(6)=NFSAVE
 
        WRITE(NF,*)
        WRITE(NF,*)
        WRITE(NF,*)
 
        IF(NXIN.EQ.0) THEN
 
          WRITE(NF,610)0,1
          DO 200 J=1,M(0)
            WRITE(NF,*)
            WRITE(NF,640)(W(JNINDX(1,I,J)),
     &                    C(NSELF(JNINDX(1,I,J))),I=1,M(1))
200       CONTINUE
 
        ELSE
 
          WRITE(NF,650)
          DO 210 IHPRF=1,NHPRF
            WRITE(NF,*)
            WRITE(NF,640)(W(IW),C(NSELF(IW)),
     &                      IW=NRFW*(IHPRF-1)+1,NRFW*IHPRF)
210       CONTINUE
          IF(NHRF*NHPRF.LT.M(1)) THEN
            WRITE(NF,*)
            WRITE(NF,660)
            DO 220 J=1,M(0)
              WRITE(NF,*)
              WRITE(NF,640)(W(JNINDX(1,I,J)),
     &                    C(NSELF(JNINDX(1,I,J))),I=NHRF*NHPRF+1,M(1))
220         CONTINUE
          ENDIF
 
        ENDIF
 
        WRITE(NF,*)
        WRITE(NF,630) 1
        WRITE(NF,*)
        WRITE(NF,640)(T(JNINDX(1,I,0)),
     &                C(NTSELF(JNINDX(1,I,0))),I=1,M(1))
 
        DO 300 IL=2,NL
 
          WRITE(NF,*)
          WRITE(NF,610)IL-1,IL
          DO 310 J=1,M(IL-1)
            WRITE(NF,*)
            WRITE(NF,640)(W(JNINDX(IL,I,J)),
     &                    C(NSELF(JNINDX(IL,I,J))),I=1,M(IL))
310       CONTINUE
 
          WRITE(NF,*)
          WRITE(NF,630)IL
          WRITE(NF,*)
          WRITE(NF,640)(T(JNINDX(IL,I,0)),
     &                  C(NTSELF(JNINDX(IL,I,0))),I=1,M(IL))
300     CONTINUE
 
      ENDIF
 
600   FORMAT(26X,'Dump of weights generated by')
610   FORMAT(21X,'Values of weights between layer',I2,' (rows) and',I2,
     &           ' (columns)')
630   FORMAT(30X,'Thresholds in layer',I2)
640   FORMAT(6(E12.4,A1))
650   FORMAT(21X,'Values of weights in receptive fields')
660   FORMAT(21X,'Values of other weights between input layer (rows)',
     &           ' and layer  1 (columns)')
 
      RETURN
 
C**** END OF JNDUMP ****************************************************
      END
C***********************************************************************
 
 
      SUBROUTINE JNERR(IERR)
 
C...JetNet subroutine ERROR
 
C...Writes out an error message and stops the execution
 
      PARAMETER(MAXV=2000,MAXM=150000,MAXI=1000,MAXO=1000)
 
      COMMON /JNDAT1/ MSTJN(40),PARJN(40),MSTJM(20),PARJM(20),
     &                OIN(MAXI),OUT(MAXO),MXNDJM
      COMMON /JNDAT2/ TINV(10),IGFN(10),ETAL(10),WIDL(10),SATM(10)
      COMMON /JNINT1/ O(MAXV),A(MAXV),D(MAXV),T(MAXV),DT(MAXV),
     &                W(MAXM),DW(MAXM),NSELF(MAXM),NTSELF(MAXV)
      COMMON /JNINT2/ M(0:10),MV0(10),MM0(10),NG(10),NL,IPOTT,
     &                ER1,ER2,SM(10),ICPON
      SAVE /JNDAT1/,/JNDAT2/,/JNINT1/,/JNINT2/
 
 
      IF (MSTJM(8).EQ.1) MSTJN(6)=MSTJM(6)
      WRITE(MSTJN(6),600) IERR
 
      IF(IERR.EQ.1) THEN
        WRITE(MSTJN(6),610) MSTJN(1)
      ELSEIF(IERR.EQ.2) THEN
        WRITE(MSTJN(6),620) MV0(NL+1),MAXV
      ELSEIF(IERR.EQ.3) THEN
        WRITE(MSTJN(6),630) MM0(NL+1),MAXM
      ELSEIF(IERR.EQ.4) THEN
        WRITE(MSTJN(6),640)
      ELSEIF(IERR.EQ.5) THEN
        WRITE(MSTJN(6),650) 'JNINDX'
      ELSEIF(IERR.EQ.6) THEN
        WRITE(MSTJN(6),650) 'JNSEFI'
      ELSEIF(IERR.EQ.7) THEN
        WRITE(MSTJN(6),660) MSTJN(10),MAXI
      ELSEIF(IERR.EQ.8) THEN
        WRITE(MSTJN(6),670) MSTJN(NL),MAXO
      ELSEIF(IERR.EQ.9) THEN
        WRITE(MSTJN(6),680) MSTJN(5)
      ELSEIF(IERR.EQ.10) THEN
        WRITE(MSTJN(6),690) (MSTJN(I),I=23,26)
      ELSEIF(IERR.EQ.11) THEN
        WRITE(MSTJN(6),700) MSTJN(10),MSTJN(23),MSTJN(24)
      ELSEIF(IERR.EQ.12) THEN
        WRITE(MSTJN(6),710)
      ELSEIF(IERR.EQ.13) THEN
        WRITE(MSTJN(6),720)
      ELSEIF(IERR.EQ.14) THEN
        WRITE(MSTJN(6),730)
      ELSEIF(IERR.EQ.15) THEN
        WRITE(MSTJN(6),740)MSTJN(3)
      ELSEIF(IERR.EQ.16) THEN
        WRITE(MSTJN(6),750)
      ELSEIF(IERR.EQ.17) THEN
        WRITE(MSTJN(6),760)
      ELSEIF(IERR.EQ.18) THEN
        WRITE(MSTJN(6),770)
      ENDIF
 
      IF(IERR.GT.0) STOP 0
 
600   FORMAT(' *** JETNET ERROR:',I2,' ***')
610   FORMAT(' Illegal number of layers (',I3,')')
620   FORMAT(' Total number of nodes (',I6,') exceeds limit (',I6,').')
630   FORMAT(' Total number of weights (',I6,') exceeds limit (',
     &I6,').')
640   FORMAT(' Number of nodes in output layer is incompatible ',/,
     &       ' with the dimension of the Potts-nodes.')
650   FORMAT(' JETNET must be initialized (with JNINIT or JNREAD) ',
     &       'before ',A6,' can be called.')
660   FORMAT(' Total number of input nodes (',I6,
     &                   ') exceeds limit (',I6,').')
670   FORMAT(' Total number of output nodes (',I6,
     &                   ') exceeds limit (',I6,').')
680   FORMAT(' Undefined updating algorithm (',I2,') chosen.')
690   FORMAT(' Inconsistent geometry for receptive fields:',/,
     &       ' (MSTJN(23) = ',I4,', MSTJN(24) = ',I4,
     &       ', MSTJN(25) = ',I4,', MSTJN(26) = ',I4,')')
700   FORMAT(' Too few input nodes (=',I4,') for receptive fields',/,
     &       ' (MSTJN(23) = ',I4,' and MSTJN(24) = ',I4,').')
710   FORMAT(' In JNSEFI: attempt to connect/disconnect unconnectable',
     &       ' nodes.')
720   FORMAT(' Cannot read file - wrong format. Try JNROLD instead.')
730   FORMAT(' Chopping not allowed on non-sigmoid functions.')
740   FORMAT(' Undefined transfer function (',I2,') in GJN.')
750   FORMAT(' Call to JNINIT after calling JMINIT')
760   FORMAT(' JNREAD cannot read data-file produced by JMDUMP')
770   FORMAT(' JNROLD cannot read data-file produced by JMDUMP')
 
      RETURN
 
C**** END OF JNERR *****************************************************
      END
C***********************************************************************
 
 
      SUBROUTINE JNFEED
 
C...JetNet subroutine FEED signal through net
 
C...Takes the the values of OIN and calculates the values of
C...the output nodes without writing to OUT
 
      PARAMETER(MAXV=2000,MAXM=150000,MAXI=1000,MAXO=1000)
 
      COMMON /JNDAT1/ MSTJN(40),PARJN(40),MSTJM(20),PARJM(20),
     &                OIN(MAXI),OUT(MAXO),MXNDJM
      COMMON /JNDAT2/ TINV(10),IGFN(10),ETAL(10),WIDL(10),SATM(10)
      COMMON /JNINT1/ O(MAXV),A(MAXV),D(MAXV),T(MAXV),DT(MAXV),
     &                W(MAXM),DW(MAXM),NSELF(MAXM),NTSELF(MAXV)
      COMMON /JNINT2/ M(0:10),MV0(10),MM0(10),NG(10),NL,IPOTT,
     &                ER1,ER2,SM(10),ICPON
      COMMON /JNINT3/ NXIN,NYIN,NXRF,NYRF,NXHRF,NYHRF,NHRF,NRFW,NHPRF
      SAVE /JNDAT1/,/JNDAT2/, /JNINT1/,/JNINT2/,/JNINT3/
 
 
C...set beta in first layer
 
      IF(TINV(1).EQ.0.0) THEN
        BETA=PARJN(3)
      ELSE
        BETA=ABS(TINV(1))
      ENDIF
 
C...calculate nodes in first layer
 
      IF(NXIN.EQ.0) THEN
 
C...normal first layer
 
        DO 100 I=1,M(1)
          A(I)=T(I)*FLOAT(NTSELF(I))
          MIJ=I-M(1)
          DO 110 J=1,M(0)
            MIJ=MIJ+M(1)
            A(I)=A(I)+W(MIJ)*OIN(J)
110       CONTINUE
          O(I)=GJN(BETA*A(I),NG(1))
100     CONTINUE
 
      ELSE
 
C...receptive fields in first layer
 
        DO 120 IHPRF=1,NHPRF
 
          DO 130 IY=1,NYHRF
            IH=IY-NYHRF+(IHPRF-1)*NHRF
            DO 140 IX=1,NXHRF
              IH=IH+NYHRF
              A(IH)=T(IH)*FLOAT(NTSELF(IH))
              DO 150 JY=1,NYRF
                IW=JY-NYRF+(IHPRF-1)*NRFW
                DO 160 JX=1,NXRF
                  IW=IW+NYRF
                  INX=IX+JX-1
                  IF(INX.GT.ABS(NXIN)) INX=INX-ABS(NXIN)
                  INY=IY+JY-1
                  IF(INY.GT.ABS(NYIN)) INY=INY-ABS(NYIN)
                  IN=(INX-1)*ABS(NYIN)+INY
                  A(IH)=A(IH)+W(IW)*OIN(IN)
160             CONTINUE
150           CONTINUE
              DO 170 IN=ABS(NXIN*NYIN)+1,M(0)
                IW=NXRF*NYRF+IN-ABS(NXIN*NYIN)+(IHPRF-1)*NRFW
                A(IH)=A(IH)+W(IW)*OIN(IN)
170           CONTINUE
              O(IH)=GJN(BETA*A(IH),NG(1))
140         CONTINUE
130       CONTINUE
120     CONTINUE
 
        DO 180 IH=NHRF*NHPRF+1,M(1)
          A(IH)=T(IH)*FLOAT(NTSELF(IH))
          IW=NHRF*NHPRF+IH-M(1)
          DO 190 IN=1,M(0)
            IW=IW+M(1)
            A(IH)=A(IH)+W(IW)*OIN(IN)
190       CONTINUE
          O(IH)=GJN(BETA*A(IH),NG(1))
180     CONTINUE
 
      ENDIF
 
C...calculate nodes in following layers
 
      DO 200 IL=2,NL
 
C...set beta in layer IL
 
       IF(TINV(IL).EQ.0.0) THEN
         BETA=PARJN(3)
       ELSE
         BETA=ABS(TINV(IL))
       ENDIF
 
C...calculate nodes in layer IL
 
       DO 210 I=1,M(IL)
         MI=MV0(IL)+I
         A(MI)=T(MI)*FLOAT(NTSELF(MI))
         MIJ=MM0(IL)-M(IL)+I
         DO 220 J=MV0(IL-1)+1,MV0(IL-1)+M(IL-1)
           MIJ=MIJ+M(IL)
           A(MI)=A(MI)+W(MIJ)*O(J)
220      CONTINUE
         O(MI)=GJN(BETA*A(MI),NG(IL))
210     CONTINUE
200   CONTINUE
 
      IF(IPOTT.LT.2) RETURN
 
C...Special treatment of output layer if Potts-nodes
 
      DO 300 I=1,M(NL)/IPOTT
        DD=0.0
        JO=MV0(NL)+(I-1)*IPOTT
        DO 310 J=1,IPOTT
          DD=DD+O(JO+J)
310     CONTINUE
        DO 320 J=1,IPOTT
          O(JO+J)=O(JO+J)/DD
320     CONTINUE
300   CONTINUE
 
      RETURN
 
C**** END OF JNFEED ****************************************************
      END
C***********************************************************************
 
 
      SUBROUTINE JNHEAD
 
C...JetNet subroutine write HEADer
 
C...Writes a header on file number NF
 
 
      PARAMETER(MAXI=1000,MAXO=1000)
 
      COMMON /JNDAT1/ MSTJN(40),PARJN(40),MSTJM(20),PARJM(20),
     &                OIN(MAXI),OUT(MAXO),MXNDJM
      SAVE /JNDAT1/
 
 
      IF(MSTJM(8).EQ.1) MSTJN(6)=MSTJM(6)
 
      WRITE(MSTJN(6),*)
      WRITE(MSTJN(6),*)
      WRITE(MSTJN(6),600)
      WRITE(MSTJN(6),610)
      WRITE(MSTJN(6),*)
 
600   FORMAT(14X,'The Lund Neural Network Program - JETNET version 2.0')
610   FORMAT(14X,'*****   Latest date of change: 15 Sept. 1991   *****')
 
      RETURN
 
C**** END OF JNHEAD ****************************************************
      END
C***********************************************************************
 
 
      INTEGER FUNCTION JNINDX(IL,I,J)
 
C...JetNet function INDeX
 
C...Gives the node vector index of node I in layer IL for J=0
C...else gives the weight vector index of weight between node
C...I of layer IL and node J of layer IL-1
 
      PARAMETER(MAXI=1000,MAXO=1000)
 
      COMMON /JNDAT1/ MSTJN(40),PARJN(40),MSTJM(20),PARJM(20),
     &                OIN(MAXI),OUT(MAXO),MXNDJM
      COMMON /JNINT2/ M(0:10),MV0(10),MM0(10),NG(10),NL,IPOTT,
     &                ER1,ER2,SM(10),ICPON
      COMMON /JNINT3/ NXIN,NYIN,NXRF,NYRF,NXHRF,NYHRF,NHRF,NRFW,NHPRF
      SAVE /JNDAT1/,/JNINT2/,/JNINT3/
 
 
      IF(MSTJN(8).EQ.0) CALL JNERR(5)
 
      IF(J.EQ.0) THEN
        JNINDX=MV0(IL)+I
      ELSE
        IF(NXIN.EQ.0.OR.IL.GT.1) THEN
          JNINDX=MM0(IL)+(J-1)*M(IL)+I
        ELSE
          IF(I.LE.NHRF*NHPRF) THEN
            IF(J.LE.ABS(NXIN*NYIN)) THEN
              IX=(I-1)/NYHRF+1
              IY=MOD(I-1,NYHRF)+1
              INX=(J-1)/ABS(NYIN)+1
              INY=MOD(J-1,ABS(NYIN))+1
              JX=INX-IX+1
              IF(JX.LE.0) JX=JX+NXRF
              IF(JX.LE.0) CALL JNERR(12)
              IF(JX.GT.NXRF) CALL JNERR(12)
              JY=INY-IY+1
              IF(JY.LE.0) JY=JY+NYRF
              IF(JY.LE.0) CALL JNERR(12)
              IF(JY.GT.NYRF) CALL JNERR(12)
              JNINDX=(JX-1)*NYRF+JY+((I-1)/NHRF)*NRFW
            ELSE
              JNINDX=NXRF*NYRF+J-ABS(NXIN*NYIN)+((I-1)/NHRF)*NRFW
            ENDIF
          ELSE
            JNINDX=NHPRF*NRFW+(J-1)*M(1)+I-NXHRF*NYHRF
          ENDIF
        ENDIF
      ENDIF
 
      RETURN
 
C**** END OF JNINDX ****************************************************
      END
C***********************************************************************
 
 
      SUBROUTINE JNINIT
 
C...JetNet subroutine INITialize net
 
C...Initializes a net according to switches and parameters in
C.../JNDAT1/ and /JNDAT2/
 
      PARAMETER(MAXV=2000,MAXM=150000,MAXI=1000,MAXO=1000)
 
      COMMON /JNDAT1/ MSTJN(40),PARJN(40),MSTJM(20),PARJM(20),
     &                OIN(MAXI),OUT(MAXO),MXNDJM
      COMMON /JNDAT2/ TINV(10),IGFN(10),ETAL(10),WIDL(10),SATM(10)
      COMMON /JNINT1/ O(MAXV),A(MAXV),D(MAXV),T(MAXV),DT(MAXV),
     &                W(MAXM),DW(MAXM),NSELF(MAXM),NTSELF(MAXV)
      COMMON /JNINT2/ M(0:10),MV0(10),MM0(10),NG(10),NL,IPOTT,
     &                ER1,ER2,SM(10),ICPON
      COMMON /JNINT3/ NXIN,NYIN,NXRF,NYRF,NXHRF,NYHRF,NHRF,NRFW,NHPRF
      SAVE /JNDAT1/,/JNDAT2/,/JNINT1/,/JNINT2/,/JNINT3/
 
 
C...Check if JMINIT has been called
 
      IF(MSTJM(8).EQ.1) CALL JNERR(16)
 
C...Set parameters in /JNINT2/
 
      CALL JNSEPA
 
C...Set initial values of weights and thresholds
 
      DO 100 IL=1,NL
 
C...Set width in this layer
 
        IF(WIDL(IL).LE.0) THEN
          WIDTH=PARJN(4)
        ELSE
          WIDTH=WIDL(IL)
        ENDIF
 
C...Initialize weights
 
        DO 110 I=MM0(IL)+1,MM0(IL+1)
          W(I)=(2.0*RLU(0)-1.0)*WIDTH
110     CONTINUE
 
C...Initialize thresholds
 
        DO 120 I=MV0(IL)+1,MV0(IL+1)
          T(I)=(2.0*RLU(0)-1.0)*WIDTH
120     CONTINUE
 
100   CONTINUE
 
      IF(NXIN.NE.0) THEN
        DO 130 IHPRF=1,NHPRF
          DO 140 I=2,NHRF
            T((IHPRF-1)*NHRF+I)=T((IHPRF-1)*NHRF+1)
140       CONTINUE
130     CONTINUE
 
        IF(MSTJN(27).LT.0) THEN
          DO 150 I=1,M(2)
            DO 160 IHPRF=1,NHPRF
              MIJ=MM0(2)+(IHPRF-1)*NHRF*M(2)+I
              SUMRFW=W(MIJ)
              DO 170 J=2,NHRF
                MIJ=MIJ+M(2)
                W(MIJ)=SUMRFW
170           CONTINUE
160         CONTINUE
150       CONTINUE
        ENDIF
 
      ENDIF
 
C...Write statistics on output file
 
      IF(MSTJN(6).LT.0) RETURN
 
      CALL JNHEAD
 
      CALL JNSTAT(1)
 
      WRITE(MSTJN(6),600)
 
600   FORMAT(22X,'Weights and thresholds set randomly')
 
      RETURN
 
C**** END OF JNINIT ****************************************************
      END
C***********************************************************************
 
 
      SUBROUTINE JNREAD(NF)
 
C...JetNet subroutine Read weights from OLD version
 
C...Reads weights, thresholds and other statistics from a file NF and
C...initializes the net
 
      PARAMETER(MAXV=2000,MAXM=150000,MAXI=1000,MAXO=1000)
 
      COMMON /JNDAT1/ MSTJN(40),PARJN(40),MSTJM(20),PARJM(20),
     &                OIN(MAXI),OUT(MAXO),MXNDJM
      COMMON /JNDAT2/ TINV(10),IGFN(10),ETAL(10),WIDL(10),SATM(10)
      COMMON /JNINT1/ O(MAXV),A(MAXV),D(MAXV),T(MAXV),DT(MAXV),
     &                W(MAXM),DW(MAXM),NSELF(MAXM),NTSELF(MAXV)
      COMMON /JNINT2/ M(0:10),MV0(10),MM0(10),NG(10),NL,IPOTT,
     &                ER1,ER2,SM(10),ICPON
      COMMON /JNINT3/ NXIN,NYIN,NXRF,NYRF,NXHRF,NYHRF,NHRF,NRFW,NHPRF
      SAVE /JNDAT1/,/JNDAT2/,/JNINT1/,/JNINT2/,/JNINT3/
 
      CHARACTER LINE*1000
 
 
      IF(NF.LT.0) THEN
 
C...unformatted read
 
        JF=-NF
        READ(JF) IVERS
        IF(IVERS.LT.0) CALL JNERR(17)
        IF(IVERS/10.NE.2) CALL JNERR(13)
        READ(JF) MSTJN,PARJN,TINV,IGFN,ETAL,WIDL,SATM
 
C...Set parameters in /JNINT2/
 
        CALL JNSEPA
 
        DO 100 I=1,MM0(NL+1)
          READ(JF) W(I)
100     CONTINUE
 
        DO 110 I=1,MV0(NL+1)
          READ(JF) T(I)
110     CONTINUE
 
        DO 120 I=1,MM0(NL+1)
          READ(JF) NSELF(I)
120     CONTINUE
 
        DO 130 I=1,MV0(NL+1)
          READ(JF) NTSELF(I)
130     CONTINUE
 
      ELSE
 
C...Formatted dump
 
        READ(NF,690)LINE
        IF (LINE(27:28).EQ.' D') CALL JNERR(17)
        READ(NF,*)
        READ(NF,*)
        READ(NF,650)FVERS
        IVERS=INT(FVERS*10.0+0.001)
        IF(IVERS/10.NE.2) CALL JNERR(13)
 
900     READ(NF,690) LINE
        IF(LINE(1:24).NE.'        I       1      2') GOTO 900
 
        NFSAVE=MSTJN(6)
 
        READ(NF,601)(MSTJN(I),I=1,6),TRN,(MSTJN(I),I=8,10)
        MSTJN(7)=INT(TRN**10+0.5)
        READ(NF,600)(MSTJN(10+I),I=1,10)
        READ(NF,600)(MSTJN(20+I),I=1,10)
        READ(NF,600)(MSTJN(30+I),I=1,10)
        READ(NF,610)(PARJN(I),I=1,10)
        READ(NF,610)(PARJN(10+I),I=1,10)
        READ(NF,610)(PARJN(20+I),I=1,10)
        READ(NF,610)(PARJN(30+I),I=1,10)
        READ(NF,600)(IGFN(I),I=1,10)
        READ(NF,610)(TINV(I),I=1,10)
        READ(NF,610)(ETAL(I),I=1,10)
        READ(NF,610)(WIDL(I),I=1,10)
        READ(NF,610)(SATM(I),I=1,10)
        READ(NF,*)
 
        MSTJN(6)=NFSAVE
 
        CALL JNSEPA
 
        READ(NF,*)
        READ(NF,*)
        READ(NF,*)
 
        IF(NXIN.EQ.0) THEN
 
          READ(NF,*)
          DO 200 J=1,M(0)
            READ(NF,*)
            READ(NF,620)(W(JNINDX(1,I,J)),
     &                    LINE(I:I),I=1,M(1))
            DO 210 I=1,M(1)
              IF(LINE(I:I).EQ.'*') THEN
                NSELF(JNINDX(1,I,J))=0
              ELSE
                NSELF(JNINDX(1,I,J))=1
              ENDIF
210         CONTINUE
 
200       CONTINUE
 
        ELSE
 
 
          READ(NF,*)
          DO 220 IHPRF=1,NHPRF
            READ(NF,*)
            READ(NF,620)(W(IW),LINE(IW:IW),
     &                     IW=NRFW*(IHPRF-1)+1,NRFW*IHPRF)
            DO 230 IW=NRFW*(IHPRF-1)+1,NRFW*IHPRF
              IF(LINE(IW:IW).EQ.'*') THEN
                NSELF(IW)=0
              ELSE
                NSELF(IW)=1
              ENDIF
230         CONTINUE
220       CONTINUE
 
          IF(NHRF*NHPRF.LT.M(1)) THEN
            READ(NF,*)
            READ(NF,*)
            DO 240 J=1,M(0)
              READ(NF,*)
              READ(NF,620)(W(JNINDX(1,I,J)),LINE(I:I),
     &                                I=NHRF*NHPRF+1,M(1))
              DO 250 I=NHRF*NHPRF+1,M(1)
                IF(LINE(I:I).EQ.'*') THEN
                  NSELF(JNINDX(1,I,J))=0
                ELSE
                  NSELF(JNINDX(1,I,J))=1
                ENDIF
250           CONTINUE
240         CONTINUE
          ENDIF
 
        ENDIF
 
        READ(NF,*)
        READ(NF,*)
        READ(NF,*)
        READ(NF,620)(T(JNINDX(1,I,0)),LINE(I:I),I=1,M(1))
        DO 260 I=1,M(1)
          IF(LINE(I:I).EQ.'*') THEN
            NTSELF(JNINDX(1,I,0))=0
          ELSE
            NTSELF(JNINDX(1,I,0))=1
          ENDIF
260     CONTINUE
 
        DO 300 IL=2,NL
 
          READ(NF,*)
          READ(NF,*)
          DO 310 J=1,M(IL-1)
            READ(NF,*)
            READ(NF,620)(W(JNINDX(IL,I,J)),LINE(I:I),I=1,M(IL))
            DO 320 I=1,M(IL)
              IF(LINE(I:I).EQ.'*') THEN
                NSELF(JNINDX(IL,I,J))=0
              ELSE
                NSELF(JNINDX(IL,I,J))=1
              ENDIF
320         CONTINUE
310       CONTINUE
 
          READ(NF,*)
          READ(NF,*)
          READ(NF,*)
          READ(NF,620)(T(JNINDX(IL,I,0)),LINE(I:I),I=1,M(IL))
          DO 330 I=1,M(IL)
            IF(LINE(I:I).EQ.'*') THEN
              NTSELF(JNINDX(IL,I,0))=0
            ELSE
              NTSELF(JNINDX(IL,I,0))=1
            ENDIF
330       CONTINUE
 
300    CONTINUE
 
      ENDIF
 
C...Write statistics on output file
 
      IF(MSTJN(6).LT.0) RETURN
 
      CALL JNHEAD
 
      CALL JNSTAT(1)
 
      WRITE(MSTJN(6),640)
 
600   FORMAT(TR11,10I7)
601   FORMAT(TR11,6I7,F7.3,3I7)
610   FORMAT(TR11,10F7.4)
620   FORMAT(6(F12.4,A1))
640   FORMAT(29X,'Weights read from file')
650   FORMAT(TR63,F2.1)
690   FORMAT(A)
 
      RETURN
 
C**** END OF JNREAD ****************************************************
      END
C***********************************************************************
 
 
      SUBROUTINE JNROLD(NF)
 
C...JetNet subroutine Read weights from OLD versions
C...(JETNET 1.0 and 1.1)
 
C...Reads weights, thresholds and other statistics from a file NF and
C...initializes the net
 
      PARAMETER(MAXV=2000,MAXM=150000,MAXI=1000,MAXO=1000)
 
      COMMON /JNDAT1/ MSTJN(40),PARJN(40),MSTJM(20),PARJM(20),
     &                OIN(MAXI),OUT(MAXO),MXNDJM
      COMMON /JNDAT2/ TINV(10),IGFN(10),ETAL(10),WIDL(10),SATM(10)
      COMMON /JNINT1/ O(MAXV),A(MAXV),D(MAXV),T(MAXV),DT(MAXV),
     &                W(MAXM),DW(MAXM),NSELF(MAXM),NTSELF(MAXV)
      COMMON /JNINT2/ M(0:10),MV0(10),MM0(10),NG(10),NL,IPOTT,
     &                ER1,ER2,SM(10),ICPON
      SAVE /JNDAT1/,/JNDAT2/,/JNINT1/,/JNINT2/
 
      CHARACTER*80 LINE
      DIMENSION MSTJNO(20),PARJNO(20)
 
 
      IF(NF.LT.0) THEN
 
C...unformatted read
 
        JF=-NF
        READ(JF) MSTJNO,PARJNO,TINV,IGFN
 
        DO 100 I=1,20
          MSTJN(I)=MSTJNO(I)
          PARJN(I)=PARJNO(I)
100     CONTINUE
 
C...Set parameters in /JNINT2/
 
        CALL JNSEPA
 
        DO 110 I=1,MM0(NL+1)
          READ(JF) W(I)
110     CONTINUE
 
        DO 120 I=1,MV0(NL+1)
          READ(JF) T(I)
120     CONTINUE
 
        DO 130 I=1,MM0(NL+1)
          READ(JF) NSELF(I)
130     CONTINUE
 
      ELSE
 
C...Formatted dump
 
        READ(NF,690)LINE
        IF (LINE(27:28).EQ.' D') CALL JNERR(18)
900     READ(NF,690) LINE
        IF(LINE(1:24).NE.'         I      1      2'.AND.
     &     LINE(1:25).NE.'          I      1      2') GOTO 900
 
        NFSAVE=MSTJN(6)
 
        READ(NF,600)(MSTJNO(I),I=1,10)
        READ(NF,600)(MSTJNO(10+I),I=1,10)
        READ(NF,610)(PARJNO(I),I=1,10)
        READ(NF,600)(IGFN(I),I=1,10)
        READ(NF,610)(TINV(I),I=1,10)
        READ(NF,*)
 
        DO 200 I=1,20
          MSTJN(I)=MSTJNO(I)
          PARJN(I)=PARJNO(I)
200     CONTINUE
 
        MSTJN(6)=NFSAVE
 
        CALL JNSEPA
 
        READ(NF,*)
        READ(NF,*)
        DO 210 IL=1,NL
 
          READ(NF,*)
          READ(NF,*)
          DO 220 J=1,M(IL-1)
            READ(NF,*)
            READ(NF,620)(W(JNINDX(IL,I,J)),LINE(I:I),I=1,M(IL))
            DO 230 I=1,M(IL)
              IF(LINE(I:I).EQ.'*') THEN
                NSELF(JNINDX(IL,I,J))=0
              ELSE
                NSELF(JNINDX(IL,I,J))=1
              ENDIF
230         CONTINUE
220       CONTINUE
 
         READ(NF,*)
         READ(NF,*)
         READ(NF,*)
         READ(NF,630)(T(JNINDX(IL,I,0)),I=1,M(IL))
210    CONTINUE
 
      ENDIF
 
C...PARJN(5) has new meaning:
 
      PARJN(5)=0
 
C...Write statistics on output file
 
      IF(MSTJN(6).LT.0) RETURN
 
      CALL JNHEAD
 
      CALL JNSTAT(1)
 
      WRITE(MSTJN(6),640)
 
600   FORMAT(TR11,10I7)
610   FORMAT(TR11,10F7.4)
620   FORMAT(10(F7.4,A1))
630   FORMAT(10F8.4)
640   FORMAT(17X,'Weights read from file produced with version 1')
690   FORMAT(A)
 
      RETURN
 
C**** END OF JNROLD ****************************************************
      END
C***********************************************************************
 
 
      SUBROUTINE JNSATM
C...JetNet subroutine SATuration Measure
 
C...Calculates the saturation measure "S" for each layer.
C...Note: The response function for the layer must be a sigmoid.
 
      PARAMETER(MAXV=2000,MAXM=150000,MAXI=1000,MAXO=1000)
 
      COMMON /JNDAT1/ MSTJN(40),PARJN(40),MSTJM(20),PARJM(20),
     &                OIN(MAXI),OUT(MAXO),MXNDJM
      COMMON /JNDAT2/ TINV(10),IGFN(10),ETAL(10),WIDL(10),SATM(10)
      COMMON /JNINT1/ O(MAXV),A(MAXV),D(MAXV),T(MAXV),DT(MAXV),
     &                W(MAXM),DW(MAXM),NSELF(MAXM),NTSELF(MAXV)
      COMMON /JNINT2/ M(0:10),MV0(10),MM0(10),NG(10),NL,IPOTT,
     &                ER1,ER2,SM(10),ICPON
      SAVE /JNDAT1/,/JNDAT2/,/JNINT1/,/JNINT2/
 
      DO 100 IL=1,NL
 
        IF (ABS(NG(IL)).EQ.1.OR.ABS(NG(IL)).EQ.5) THEN
 
          SUM=0.0
          DO 110 I=1,M(IL)
            MI=MV0(IL)+I
            SUM=SUM+(1.-2.*O(MI))**2
110       CONTINUE
          SM(IL)=SM(IL)+SUM/FLOAT(M(IL))
 
        ELSEIF (ABS(NG(IL)).EQ.2) THEN
 
          SUM=0.0
          DO 120 I=1,M(IL)
            MI=MV0(IL)+I
            SUM=SUM+O(MI)**2
120       CONTINUE
          SM(IL)=SM(IL)+SUM/FLOAT(M(IL))
 
        ELSE
          SM(IL)=0.0
        ENDIF
 
100   CONTINUE
 
      RETURN
 
C**** END OF JNSATM ****************************************************
      END
C***********************************************************************
 
 
      SUBROUTINE JNSEFI(ILA,I1,I2,J1,J2,NO)
 
C...JetNet subroutine SElect FIelds
 
C...Switches the updating of the weights between nodes I1 to I2 in layer
C...ILA and nodes J1 to J2 in layer ILA-1 on or off according to NO. If
C...NO<=0 updating is turned off else it is turned on. In addition if
C...NO=0 the weight is set to zero and if NO=1 the weight is
C...reinitialized.
 
      PARAMETER(MAXV=2000,MAXM=150000,MAXI=1000,MAXO=1000)
 
      COMMON /JNDAT1/ MSTJN(40),PARJN(40),MSTJM(20),PARJM(20),
     &                OIN(MAXI),OUT(MAXO),MXNDJM
      COMMON /JNDAT2/ TINV(10),IGFN(10),ETAL(10),WIDL(10),SATM(10)
      COMMON /JNINT1/ O(MAXV),A(MAXV),D(MAXV),T(MAXV),DT(MAXV),
     &                W(MAXM),DW(MAXM),NSELF(MAXM),NTSELF(MAXV)
      COMMON /JNINT2/ M(0:10),MV0(10),MM0(10),NG(10),NL,IPOTT,
     &                ER1,ER2,SM(10),ICPON
      SAVE /JNDAT1/,/JNDAT2/,/JNINT1/,/JNINT2/
 
 
      IF(MSTJN(8).EQ.0) CALL JNERR(6)
 
      IF=MAX(I1,1)
      IL=MIN(I2,M(ILA))
 
      IF(WIDL(ILA).LE.0.0) THEN
        WIDTH=PARJN(4)
      ELSE
        WIDTH=WIDL(ILA)
      ENDIF
 
      IF(J1.NE.0.OR.J2.NE.0) THEN
 
        JF=MAX(J1,1)
        JL=MIN(J2,M(ILA-1))
 
        DO 100 II=IF,IL
          DO 110 JJ=JF,JL
            IF(NO.GT.0) THEN
              NSELF(JNINDX(ILA,II,JJ))=1
            ELSE
              NSELF(JNINDX(ILA,II,JJ))=0
            ENDIF
            IF(NO.EQ.1) THEN
              W(JNINDX(ILA,II,JJ))=(2.0*RLU(0)-1.0)*WIDTH
            ELSEIF(NO.EQ.0) THEN
              W(JNINDX(ILA,II,JJ))=0.0
            ENDIF
110       CONTINUE
100     CONTINUE
 
      ELSE
 
        DO 200 II=IF,IL
          IF(NO.GT.0) THEN
            NTSELF(JNINDX(ILA,II,0))=1
          ELSE
            NTSELF(JNINDX(ILA,II,0))=0
          ENDIF
          IF(NO.EQ.1) THEN
            T(JNINDX(ILA,II,0))=(2.0*RLU(0)-1.0)*WIDTH
          ELSEIF(NO.EQ.0) THEN
            T(JNINDX(ILA,II,0))=0.0
          ENDIF
200     CONTINUE
 
      ENDIF
 
      RETURN
 
C**** END OF JNSEFI ****************************************************
      END
C***********************************************************************
 
 
      SUBROUTINE JNSEPA
 
C...JetNet subroutine SEt PArameters
 
C...Sets parameters in /JNINT2/
 
      PARAMETER(MAXV=2000,MAXM=150000,MAXI=1000,MAXO=1000)
 
      COMMON /JNDAT1/ MSTJN(40),PARJN(40),MSTJM(20),PARJM(20),
     &                OIN(MAXI),OUT(MAXO),MXNDJM
      COMMON /JNDAT2/ TINV(10),IGFN(10),ETAL(10),WIDL(10),SATM(10)
      COMMON /JNINT1/ O(MAXV),A(MAXV),D(MAXV),T(MAXV),DT(MAXV),
     &                W(MAXM),DW(MAXM),NSELF(MAXM),NTSELF(MAXV)
      COMMON /JNINT2/ M(0:10),MV0(10),MM0(10),NG(10),NL,IPOTT,
     &                ER1,ER2,SM(10),ICPON
      COMMON /JNINT3/ NXIN,NYIN,NXRF,NYRF,NXHRF,NYHRF,NHRF,NRFW,NHPRF
      SAVE /JNDAT1/,/JNDAT2/,/JNINT1/,/JNINT2/,/JNINT3/
 
 
C...last layer:
 
      NL=MSTJN(1)-1
      IF(NL.GT.10.OR.NL.LT.1) CALL JNERR(1)
 
C...number of nodes in each layer
 
      IF(MSTJN(10).GT.MAXI) CALL JNERR(7)
      IF(MSTJN(10+NL).GT.MAXO) CALL JNERR(8)
 
      IF(MSTJN(23).NE.0) THEN
 
C...receptive fields will be used, check consistency and set indexes
 
        IF(MSTJN(24).EQ.0.OR.MSTJN(25).LE.0.OR.MSTJN(26).LE.0)
     &      CALL JNERR(10)
 
        NXIN=MSTJN(23)
        NYIN=MSTJN(24)
        NXRF=MSTJN(25)
        NYRF=MSTJN(26)
        NHPRF=ABS(MSTJN(27))
        IF(MSTJN(10).LT.ABS(NXIN*NYIN)) CALL JNERR(11)
        if(NXRF.GT.ABS(NXIN).OR.NYRF.GT.ABS(NYIN)) CALL JNERR(11)
 
        IF(NXIN.GT.0) THEN
          NXHRF=NXIN-NXRF+1
        ELSE
          NXHRF=-NXIN
        ENDIF
 
        IF(NYIN.GT.0) THEN
          NYHRF=NYIN-NYRF+1
        ELSE
          NYHRF=-NYIN
        ENDIF
 
        NHRF=NXHRF*NYHRF
        MSTJN(11)=MAX(MSTJN(11),NHRF*NHPRF)
 
        NRFW=NXRF*NYRF+MSTJN(10)-ABS(NXIN*NYIN)
        NRFLW=NRFW*NHPRF+(MSTJN(11)-NHRF*NHPRF)*MSTJN(10)
 
      ELSE
 
        NXIN=0
        NYIN=0
        NXRF=0
        NYRF=0
        NXHRF=0
        NYHRF=0
        NHRF=0
        NRFW=0
        NRFLW=0
 
      ENDIF
 
      DO 100 IL=0,NL
        M(IL)=MSTJN(10+IL)
100   CONTINUE
 
C...offset index in node vectors and weight vectors
 
      MV0(1)=0
      MM0(1)=0
      MV0(2)=M(1)
      MM0(2)=M(0)*M(1)
      IF(NXIN.NE.0) MM0(2)=NRFLW
 
      DO 110 IL=3,NL+1
        MV0(IL)=MV0(IL-1)+M(IL-1)
        MM0(IL)=MM0(IL-1)+M(IL-1)*M(IL-2)
110   CONTINUE
 
      IF(MV0(NL+1).GT.MAXV) CALL JNERR(2)
      IF(MM0(NL+1).GT.MAXM) CALL JNERR(3)
 
C...check Potts-nodes
 
      IPOTT=MSTJN(4)
      IF(IPOTT.GE.2) THEN
        IF(MOD(M(NL),IPOTT).NE.0) CALL JNERR(4)
        IGFN(NL)=3
      ENDIF
      IF(IPOTT.EQ.1) IGFN(NL)=5
 
C...set sigmoid functions to use
 
      DO 120 IL=1,NL
        IF(IGFN(IL).EQ.0) THEN
          NG(IL)=MSTJN(3)
        ELSE
          NG(IL)=IGFN(IL)
        ENDIF
120   CONTINUE
 
      DO 200 I=1,MV0(NL+1)
        DT(I)=0.0
        NTSELF(I)=1
200   CONTINUE
 
      DO 210 I=1,MM0(NL+1)
        DW(I)=0.0
        NSELF(I)=1
210   CONTINUE
 
C...set precision chopping
 
      IF((MSTJN(28).GT.0).OR.(MSTJN(29).GT.0).OR.
     &(MSTJN(30).GT.0)) ICPON=1
 
      MSTJN(8)=1
 
      RETURN
 
C**** END OF JNSEPA ****************************************************
      END
C***********************************************************************
 
 
      SUBROUTINE JNSTAT(IS)
 
C...JetNet subroutine output STATistics
 
C...Statistics chosen by IS is written on the default file
 
      PARAMETER(MAXV=2000,MAXM=150000,MAXI=1000,MAXO=1000)
 
      COMMON /JNDAT1/ MSTJN(40),PARJN(40),MSTJM(20),PARJM(20),
     &                OIN(MAXI),OUT(MAXO),MXNDJM
      COMMON /JNDAT2/ TINV(10),IGFN(10),ETAL(10),WIDL(10),SATM(10)
      COMMON /JNINT1/ O(MAXV),A(MAXV),D(MAXV),T(MAXV),DT(MAXV),
     &                W(MAXM),DW(MAXM),NSELF(MAXM),NTSELF(MAXV)
      COMMON /JNINT2/ M(0:10),MV0(10),MM0(10),NG(10),NL,IPOTT,
     &                ER1,ER2,SM(10),ICPON
      COMMON /JNINT3/ NXIN,NYIN,NXRF,NYRF,NXHRF,NYHRF,NHRF,NRFW,NHPRF
      SAVE /JNDAT1/,/JNDAT2/,/JNINT1/,/JNINT2/,/JNINT3/
 
 
      IF(IS.EQ.1) THEN
C...Write out number of layers, units and receptive field status
        WRITE(MSTJN(6),*)
        WRITE(MSTJN(6),600) NL+1
        WRITE(MSTJN(6),615) M(NL),NL
        DO 100 IL=NL-1,1,-1
          WRITE(MSTJN(6),620) M(IL),IL
100     CONTINUE
        WRITE(MSTJN(6),610) M(0)
        WRITE(MSTJN(6),*)
        IF(IPOTT.GT.1) WRITE(MSTJN(6),630)IPOTT
        IF(MSTJN(23).NE.0) THEN
          WRITE(MSTJN(6),631) ABS(MSTJN(23)*MSTJN(24)),ABS(MSTJN(23)),
     &    ABS(MSTJN(24)),MSTJN(25),MSTJN(26),ABS(MSTJN(27))
        ENDIF
        IF(MSTJN(23).LT.0) WRITE(MSTJN(6),632)
        IF(MSTJN(24).LT.0) WRITE(MSTJN(6),633)
        IF(MSTJN(27).LT.0) WRITE(MSTJN(6),634)
        IF(IPOTT.EQ.1) WRITE(MSTJN(6),635)
        IF(MSTJN(5).EQ.1) WRITE(MSTJN(6),640)
        IF(MSTJN(5).EQ.2) WRITE(MSTJN(6),645)
        WRITE(MSTJN(6),*)
 
      ELSEIF(IS.EQ.2) THEN
C...Write out values of parameters and switches
        WRITE(MSTJN(6),*)
        WRITE(MSTJN(6),650)
        WRITE(MSTJN(6),*)
        WRITE(MSTJN(6),660)'I ',(I,I=1,10)
        WRITE(MSTJN(6),661)'MSTJN (I)',(MSTJN(I),I=1,6),
     &            LOG10(FLOAT(MSTJN(7))),(MSTJN(I),I=8,10)
        WRITE(MSTJN(6),660)'(10+I)',(MSTJN(10+I),I=1,10)
        WRITE(MSTJN(6),660)'(20+I)',(MSTJN(20+I),I=1,10)
        WRITE(MSTJN(6),660)'(30+I)',(MSTJN(30+I),I=1,10)
        WRITE(MSTJN(6),670)'PARJN (I)',(PARJN(I),I=1,10)
        WRITE(MSTJN(6),670)'(10+I)',(PARJN(10+I),I=1,10)
        WRITE(MSTJN(6),670)'(20+I)',(PARJN(20+I),I=1,10)
        WRITE(MSTJN(6),670)'(30+I)',(PARJN(30+I),I=1,10)
        WRITE(MSTJN(6),660)'IGFN (I)',(IGFN(I),I=1,10)
        WRITE(MSTJN(6),670)'TINV (I)',(TINV(I),I=1,10)
        WRITE(MSTJN(6),670)'ETAL (I)',(ETAL(I),I=1,10)
        WRITE(MSTJN(6),670)'WIDL (I)',(WIDL(I),I=1,10)
        WRITE(MSTJN(6),670)'SATM (I)',(SATM(I),I=1,10)
        WRITE(MSTJN(6),*)
 
      ELSE
C...Write out time factor for net
        NWFAC=0
        IF(NXIN.EQ.0) THEN
          NWFAC=MM0(NL+1)+MV0(NL+1)
          NWSUM=NWFAC
        ELSE
          NWFAC=MM0(NL+1)+MV0(NL+1)-MM0(2)+NHRF*NRFW*NHPRF
     &          +(MSTJN(11)-NHRF*NHPRF)*MSTJN(10)
          NWSUM=MM0(NL+1)+MV0(NL+1)-MM0(2)+NRFW*NHPRF-(NHRF-1)*NHPRF
     &          +(MSTJN(11)-NHRF*NHPRF)*MSTJN(10)
          IF(MSTJN(27).LT.0) NWSUM=NWSUM-(NHRF-1)*NHPRF*MSTJN(12)
        ENDIF
        WRITE(MSTJN(6),680) NWFAC
        WRITE(MSTJN(6),690) NWSUM
      ENDIF
 
600   FORMAT(22X,'Initialized for a',I2,' layered net with')
610   FORMAT(22X,I3,' nodes in layer number 0 (input layer)')
615   FORMAT(22X,I3,' nodes in layer number',I2,' (output layer)')
620   FORMAT(22X,I3,' nodes in layer number',I2)
630   FORMAT(5X,'with',I3,'-dimensional Potts nodes in output layer.')
631   FORMAT(5X,'receptive fields in first layer assuming the ',I4,
     &   ' first nodes in the',/,
     &   5X,'input layer are organised in a plane of ',
     &   I3,'*',I3,' nodes, where the',/,
     &   5X,'receptive field nodes scan ',
     &   I3,'*',I3,' input nodes each with ',I3,' hidden'/,
     &   5X,'nodes per field.')
632   FORMAT(5X,'The input layer is assumed to be cyclic ',
     &       'in the x-direction.')
633   FORMAT(5X,'The input layer is assumed to be cyclic ',
     &       'in the y-direction.')
634   FORMAT(5X,'The weights from equivalent nodes with receptive ',
     &           'fields are clamped.')
635   FORMAT(5X,'Using entropy error.')
640   FORMAT(5X,'Using Manhattan updating.')
645   FORMAT(5X,'Using Langevin updating.')
650   FORMAT(18X,'Values of parameters and switches in JETNET')
660   FORMAT(A10,10I7)
661   FORMAT(A10,6I7,F7.3,3I7)
670   FORMAT(A10,10F7.4)
680   FORMAT(5X,'Time factor for this net:',I10)
690   FORMAT(5X,'Effective number of weights:',I7)
      RETURN
 
C**** END OF JNSTAT ****************************************************
      END
C***********************************************************************
 
 
      SUBROUTINE JNTEST
 
C...JetNet subroutine TEST
 
C...Sets the values of OUT according to given pattern in OIN and
C...current weights
 
      PARAMETER(MAXV=2000,MAXM=150000,MAXI=1000,MAXO=1000)
 
      COMMON /JNDAT1/ MSTJN(40),PARJN(40),MSTJM(20),PARJM(20),
     &                OIN(MAXI),OUT(MAXO),MXNDJM
      COMMON /JNDAT2/ TINV(10),IGFN(10),ETAL(10),WIDL(10),SATM(10)
      COMMON /JNINT1/ O(MAXV),A(MAXV),D(MAXV),T(MAXV),DT(MAXV),
     &                W(MAXM),DW(MAXM),NSELF(MAXM),NTSELF(MAXV)
      COMMON /JNINT2/ M(0:10),MV0(10),MM0(10),NG(10),NL,IPOTT,
     &                ER1,ER2,SM(10),ICPON
      SAVE /JNDAT1/,/JNDAT2/,/JNINT1/,/JNINT2/
 
 
      CALL JNFEED
 
      DO 100 I=1,M(NL)
        OUT(I)=O(JNINDX(NL,I,0))
100   CONTINUE
 
      RETURN
 
C**** END OF JNTEST ****************************************************
      END
C***********************************************************************
 
 
      SUBROUTINE JNTRAL
 
C...JetNet subroutine TRaining ALgorithm
 
C...Uses a back-propagation algorithm to train the net
 
      PARAMETER(MAXV=2000,MAXM=150000,MAXI=1000,MAXO=1000)
 
      COMMON /JNDAT1/ MSTJN(40),PARJN(40),MSTJM(20),PARJM(20),
     &                OIN(MAXI),OUT(MAXO),MXNDJM
      COMMON /JNDAT2/ TINV(10),IGFN(10),ETAL(10),WIDL(10),SATM(10)
      COMMON /JNINT1/ O(MAXV),A(MAXV),D(MAXV),T(MAXV),DT(MAXV),
     &                W(MAXM),DW(MAXM),NSELF(MAXM),NTSELF(MAXV)
      COMMON /JNINT2/ M(0:10),MV0(10),MM0(10),NG(10),NL,IPOTT,
     &                ER1,ER2,SM(10),ICPON
      SAVE /JNDAT1/,/JNDAT2/,/JNINT1/,/JNINT2/
 
 
      MSTJN(7)=MSTJN(7)+1
 
      CALL JNFEED
      CALL JNDELT
 
      ERR=0.0
 
      DO 100 I=1,M(NL)
        ERR=ERR+(OUT(I)-O(JNINDX(NL,I,0)))**2
        OUT(I)=O(JNINDX(NL,I,0))
100   CONTINUE
 
      ERR=ERR/FLOAT(M(NL))
      PARJN(7)=ERR
      ER1=ER1+ERR
      ER2=ER2+ERR
 
      IF (MSTJN(22).NE.0) CALL JNSATM
 
      IF(MOD(MSTJN(7),MSTJN(2)).NE.0) RETURN
 
C...update only every MSTJN(2) calls
 
      PARJN(8)=ER1/FLOAT(MSTJN(2))
      ER1=0.0
 
 
      IF(MSTJN(21).GT.0) THEN
 
C...Include pruning factors
 
        DO 110 I=1,MM0(NL+1)
          DW(I)=DW(I)-2.0*PARJN(14)*PARJN(18)**2*PARJN(1)*W(I)/
     &          ((1.0+(W(I)/PARJN(18))**2)**2)
110     CONTINUE
 
        DO 120 I=1,MV0(NL+1)
          DT(I)=DT(I)-2.0*PARJN(14)*PARJN(18)*PARJN(1)*T(I)/
     &          ((1.0+(T(I)/PARJN(18))**2)**2)
120     CONTINUE
 
      ENDIF
 
      IF(MSTJN(5).EQ.0) THEN
 
C...Normal updating:
 
        DO 200 I=1,MM0(NL+1)
          W(I)=(1.0-PARJN(5))*W(I)+DW(I)*FLOAT(NSELF(I))
          DW(I)=DW(I)*PARJN(2)
200     CONTINUE
 
        DO 210 I=1,MV0(NL+1)
          T(I)=(1.0-PARJN(5))*T(I)+DT(I)*FLOAT(NTSELF(I))
          DT(I)=DT(I)*PARJN(2)
210     CONTINUE
 
      ELSEIF(MSTJN(5).EQ.1) THEN
 
C...Manhattan updating:
 
        DO 300 I=1,MM0(NL+1)
          W(I)=(1.0-PARJN(5))*W(I)+SIGN(PARJN(1),DW(I))*FLOAT(NSELF(I))
          DW(I)=DW(I)*PARJN(2)
300     CONTINUE
 
        DO 310 I=1,MV0(NL+1)
          T(I)=(1.0-PARJN(5))*T(I)+SIGN(PARJN(1),DT(I))*FLOAT(NTSELF(I))
          DT(I)=DT(I)*PARJN(2)
310     CONTINUE
 
      ELSEIF(MSTJN(5).EQ.2) THEN
 
C...Langevin updating:
 
        DO 400 I=1,MM0(NL+1)
          W(I)=W(I)+DW(I)*FLOAT(NSELF(I))+GAUSJN(0)*PARJN(6)
          DW(I)=DW(I)*PARJN(2)
400     CONTINUE
 
        DO 410 I=1,MV0(NL+1)
          T(I)=T(I)+DT(I)*FLOAT(NTSELF(I))+GAUSJN(0)*PARJN(6)
          DT(I)=DT(I)*PARJN(2)
410     CONTINUE
 
      ELSE
 
        CALL JNERR(9)
 
      ENDIF
 
C...do fixed precision weights
 
      IF(ICPON.EQ.1) CALL JNCHOP(0)
 
C...Scale temperature
 
      IF(MSTJN(22).GE.0) THEN
 
        SCALE=PARJN(13)**(1.0/FLOAT(MSTJN(9)))
        PARJN(3)=PARJN(3)/SCALE
        DO 500 I=1,NL
          TINV(I)=TINV(I)/SCALE
500     CONTINUE
 
      ENDIF
 
      IF(MOD(MSTJN(7),MSTJN(2)*MSTJN(9)).NE.0) RETURN
 
C...Update some parameters every epoch
 
      OLDE=PARJN(9)
      PARJN(9)=ER2/FLOAT(MSTJN(2)*MSTJN(9))
      ER2=0.0
 
      IF (MSTJN(21).GT.0) THEN
 
C...Update pruning parameters
 
        PARJN(10)=PARJN(16)*PARJN(10)+(1.-PARJN(16))*PARJN(9)
        IF(PARJN(9).LT.OLDE.OR.PARJN(9).LT.PARJN(19)) THEN
          PARJN(14)=PARJN(14)+PARJN(15)
        ELSEIF(PARJN(9).LT.PARJN(10)) THEN
          PARJN(14)=PARJN(14)-PARJN(15)
        ELSE
          PARJN(14)=PARJN(16)*PARJN(14)
        ENDIF
      ENDIF
 
      IF (MSTJN(22).NE.0) THEN
 
C...Calculate saturation measures
 
        DO 510 IL=1,NL
          SATM(IL)=SM(IL)/FLOAT(MSTJN(2)*MSTJN(9))
          SM(IL)=0.0
510     CONTINUE
      ENDIF
 
      IF(MSTJN(22).LT.0) THEN
 
        DO 520 I=1,NL
 
          IF(TINV(I).GE.0.0) GOTO 520
 
          IF(SATM(I).GT.0.75) THEN
            TINV(I)=TINV(I)/(1.0+16.0*(PARJN(13)-1.0)*(SATM(I)-0.5)**2)
            GOTO 530
          ELSEIF(SATM(I).LT.0.25) THEN
            TINV(I)=TINV(I)*(1.0+16.0*(PARJN(13)-1.0)*(0.5-SATM(I))**2)
            GOTO 530
          ENDIF
520     CONTINUE
 
530   ENDIF
 
 
C...Scale eta, and alpha
 
      PARJN(1)=PARJN(1)*PARJN(11)
      DO 540 I=1,10
        ETAL(I)=ETAL(I)*PARJN(11)
540   CONTINUE
      PARJN(2)=MAX(0.0,1.0-(1.0-PARJN(2))*PARJN(12))
 
      RETURN
 
C**** END OF JNTRAL ****************************************************
      END
C***********************************************************************
 
C**********************************************************************C
C PART TWO: SELF-ORGANIZING MAP NETWORK                                C
C**********************************************************************C
 
 
      REAL FUNCTION GJM(X,N)
C...JetMap function G.
 
C...Gives response function N with argument X.
 
      PARAMETER(MAXI=1000,MAXO=1000)
 
      COMMON /JNDAT1/ MSTJN(40),PARJN(40),MSTJM(20),PARJM(20),
     &                OIN(MAXI),OUT(MAXO),MXNDJM
      SAVE /JNDAT1/
 
 
      IF(N.EQ.1) THEN
        GJM=0.5*(1.0+TANH(X))
      ELSEIF(N.EQ.2) THEN
        GJM=EXP(MAX(-50.0,MIN(-X,50.0)))
      ELSE
        MSTJM(3)=N
        CALL JMERR(11)
      ENDIF
 
      RETURN
 
C**** END OF GJM *******************************************************
      END
C***********************************************************************
 
 
      SUBROUTINE JMDUMP(NF)
 
C...JetMap subroutine DUMP weights
 
C...Dumps weights and other characteristics of the
C...net to file NF for use in other programs.
 
      PARAMETER(MAXV=2000,MAXM=150000,MAXI=1000,MAXO=1000,MAXD=2)
 
      COMMON /JNDAT1/ MSTJN(40),PARJN(40),MSTJM(20),PARJM(20),
     &                OIN(MAXI),OUT(MAXO),MXNDJM
      COMMON /JNINT1/ O(MAXV),A(MAXV),D(MAXV),T(MAXV),DT(MAXV),
     &                W(MAXM),DW(MAXM),NSELF(MAXM),NTSELF(MAXV)
      COMMON /JMINT1/ NDIM,ISW(10),NODES(0:MAXD+1),NBO
      SAVE /JNDAT1/,/JNINT1/,/JMINT1/
 
      DIMENSION INDW(MAXV)
      EQUIVALENCE (NTSELF(1),INDW(1))
 
 
      IF (MSTJM(8).NE.1) CALL JMERR(8)
 
      IF(NF.LT.0) THEN
 
C...Unformatted dump
 
        JF=-NF
        WRITE(JF) -20
        WRITE(JF) MSTJM,PARJM
 
        DO 100 IW=1,INDW(NODES(MAXD+1)+1)-1
          WRITE(JF) W(IW)
100     CONTINUE
 
      ELSE
 
C...Formatted dump
 
        NFSAVE=MSTJM(6)
        MSTJM(6)=NF
 
        WRITE(NF,600)
        CALL JNHEAD
        CALL JMSTAT(1)
        CALL JMSTAT(2)
 
        MSTJM(6)=NFSAVE
 
        DO 200 INOD=1,NODES(MAXD+1)
          WRITE(NF,*)
          CALL JMINDX(INOD,I,J)
          IF (NDIM.EQ.1) THEN
            WRITE(NF,610)I
          ELSE
            WRITE(NF,620)I,J
          ENDIF
 
          IW=INDW(INOD)-1
          WRITE(NF,630)(W(IW+K),K=1,NODES(0))
 
200    CONTINUE
 
      ENDIF
 
600   FORMAT(26X,' Dump of weights generated by')
610   FORMAT('Unit ',I2)
620   FORMAT('Unit (',I2,',',I2,')')
630   FORMAT(10(F8.4,1X))
 
      RETURN
 
C**** END OF JMDUMP ****************************************************
      END
C***********************************************************************
 
 
      SUBROUTINE JMERR(IERR)
C...JetMap subroutine ERRor.
 
C...Writes out an error message and stops the execution.
 
      PARAMETER(MAXV=2000,MAXM=150000,MAXI=1000,MAXO=1000,MAXD=2)
 
      COMMON /JNDAT1/ MSTJN(40),PARJN(40),MSTJM(20),PARJM(20),
     &                OIN(MAXI),OUT(MAXO),MXNDJM
      COMMON /JNINT1/ O(MAXV),A(MAXV),D(MAXV),T(MAXV),DT(MAXV),
     &                W(MAXM),DW(MAXM),NSELF(MAXM),NTSELF(MAXV)
      COMMON /JMINT1/ NDIM,ISW(10),NODES(0:MAXD+1),NBO
      SAVE /JNDAT1/,/JNINT1/,/JMINT1/
 
      DIMENSION INDW(MAXV)
      EQUIVALENCE (NTSELF(1),INDW(1))
 
 
      IF (MSTJN(8).EQ.1) MSTJM(6)=MSTJN(6)
      WRITE(MSTJM(6),600)IERR
 
      IF (IERR.EQ.1) THEN
        WRITE(MSTJM(6),610)MSTJM(1)
      ELSEIF (IERR.EQ.2) THEN
        WRITE(MSTJM(6),620)MSTJM(10),MAXI
      ELSEIF (IERR.EQ.3) THEN
        WRITE(MSTJM(6),630)ABS(MSTJM(11)*MSTJM(12)),MAXO
      ELSEIF (IERR.EQ.4) THEN
        WRITE(MSTJM(6),640)NODES(MAXD+1)*(NODES(0)+1),MAXM
      ELSEIF (IERR.EQ.5) THEN
        WRITE(MSTJM(6),650)'JMTEST'
      ELSEIF (IERR.EQ.6) THEN
        WRITE(MSTJM(6),650)'JMTRAL'
      ELSEIF (IERR.EQ.7) THEN
        WRITE(MSTJM(6),650)'JMINDX'
      ELSEIF (IERR.EQ.8) THEN
        WRITE(MSTJM(6),650)'JMDUMP'
      ELSEIF (IERR.EQ.9) THEN
        WRITE(MSTJM(6),650)'JMINWE'
      ELSEIF (IERR.EQ.10) THEN
        WRITE(MSTJM(6),660)
      ELSEIF (IERR.EQ.11) THEN
        WRITE(MSTJM(6),670)MSTJM(3)
      ELSEIF (IERR.EQ.12) THEN
        WRITE(MSTJM(6),680)
      ELSEIF (IERR.EQ.13) THEN
        WRITE(MSTJM(6),690)
      ELSEIF (IERR.EQ.14) THEN
        WRITE(MSTJM(6),650)'JMNBHD'
      ENDIF
 
      IF (IERR.GT.0) STOP 0
 
600   FORMAT(' *** JETMAP ERROR:',I2,' ***')
610   FORMAT(' Illegal number of dimensions (',I2,')')
620   FORMAT(' Total number of input nodes (',I6,') exceeds limit (',
     &I6,')')
630   FORMAT(' Total number of network nodes (',I6,
     &') exceeds limit (',I6,')')
640   FORMAT(' The number of weights (',I6,') exceeds limit (',I5,')')
650   FORMAT(' Network must be initialized (with JMINIT or JMREAD) ',
     &'before ',A6,' can be called')
660   FORMAT(' Call to JMINIT after calling JNINIT')
670   FORMAT(' Undefined response function (',I2,') in GJM.')
680   FORMAT(' JMREAD cannot read data-file produced by JNDUMP')
690   FORMAT(' Too many warnings issued by JETMAP')
 
      RETURN
 
C**** END OF JMERR *****************************************************
      END
C***********************************************************************
 
 
      SUBROUTINE JMFEED
C...JetMap subroutine FEED signal to net.
 
C...Feeds the input signal into the network and calculates
C...MXNDJM and DW.
 
      PARAMETER(MAXV=2000,MAXM=150000,MAXI=1000,MAXO=1000,MAXD=2)
      PARAMETER(BIAS=0.5)
 
      COMMON /JNDAT1/ MSTJN(40),PARJN(40),MSTJM(20),PARJM(20),
     &                OIN(MAXI),OUT(MAXO),MXNDJM
      COMMON /JNINT1/ O(MAXV),A(MAXV),D(MAXV),T(MAXV),DT(MAXV),
     &                W(MAXM),DW(MAXM),NSELF(MAXM),NTSELF(MAXV)
      COMMON /JMINT1/ NDIM,ISW(10),NODES(0:MAXD+1),NBO
      SAVE /JNDAT1/,/JNINT1/,/JMINT1/
 
      DIMENSION INDW(MAXV)
      EQUIVALENCE (NTSELF(1),INDW(1))
 
 
      MXNDJM=0
 
      IF (ISW(1).EQ.1) THEN
C *** Sigmoidal unit ***
 
        ODUM=0.0
        IW=0
        DO 100 J=1,NODES(MAXD+1)
          RSUM=0.0
          DO 110 K=1,NODES(0)
            IW=IW+1
            DW(IW)=OIN(K)-W(IW)
            RSUM=RSUM+OIN(K)*W(IW)
110       CONTINUE
          O(J)=GJM((RSUM-BIAS)*PARJM(3),ISW(1))
          IF (O(J).GT.ODUM) THEN
            ODUM=O(J)
            MXNDJM=J
          ENDIF
100     CONTINUE
 
      ELSEIF(ISW(1).EQ.2) THEN
C *** Gaussian unit ***
 
        ODUM=EXP(-49.0)
        IW=0
        DO 200 J=1,NODES(MAXD+1)
          RSUM=0.0
          DO 210 K=1,NODES(0)
            IW=IW+1
            DW(IW)=OIN(K)-W(IW)
            RSUM=RSUM+DW(IW)**2
210       CONTINUE
          O(J)=GJM(RSUM*PARJM(3),ISW(1))
          IF (O(J).GT.ODUM) THEN
            ODUM=O(J)
            MXNDJM=J
          ENDIF
200     CONTINUE
 
      ENDIF
 
      IF (MXNDJM.EQ.0) CALL JMWARN(1)
 
      RETURN
 
C**** END OF JMFEED ****************************************************
      END
C***********************************************************************
 
 
      SUBROUTINE JMINDX(INOD,I,J)
C...JetMap subroutine INDeX.
 
C...If INOD>0 it returns the (I,J)-coordinates for INOD, if
C...INOD=0 it returns the INOD-number (in H array) for unit (I,J)
C...in net.
 
      PARAMETER(MAXV=2000,MAXM=150000,MAXI=1000,MAXO=1000,MAXD=2)
 
      COMMON /JNDAT1/ MSTJN(40),PARJN(40),MSTJM(20),PARJM(20),
     &                OIN(MAXI),OUT(MAXO),MXNDJM
      COMMON /JNINT1/ O(MAXV),A(MAXV),D(MAXV),T(MAXV),DT(MAXV),
     &                W(MAXM),DW(MAXM),NSELF(MAXM),NTSELF(MAXV)
      COMMON /JMINT1/ NDIM,ISW(10),NODES(0:MAXD+1),NBO
      SAVE /JNDAT1/,/JNINT1/,/JMINT1/
 
      DIMENSION INDW(MAXV)
      EQUIVALENCE (NTSELF(1),INDW(1))
 
 
      IF (MSTJM(8).NE.1) CALL JMERR(7)
 
      IF (INOD.GT.0) THEN
C...INOD -> (I,J)
 
        IF (NDIM.EQ.1) THEN
          J=1
          I=INOD
        ELSEIF (NDIM.EQ.2) THEN
          J=MOD(INOD,NODES(2))
          I=INOD/NODES(2)+1
          IF (J.EQ.0) THEN
            J=NODES(2)
            I=I-1
          ENDIF
        ENDIF
 
      ELSEIF (INOD.EQ.0) THEN
C...(I,J) -> INOD
 
        IF (NDIM.EQ.1) THEN
          INOD=I
        ELSEIF (NDIM.EQ.2) THEN
          INOD=(I-1)*NODES(2)+J
        ENDIF
 
      ENDIF
 
      RETURN
 
C**** END OF JMINDX ****************************************************
      END
C***********************************************************************
 
 
      SUBROUTINE JMINIT
C...JetMap subroutine INITialize net.
 
C...Initializes the net according to switches and parameters in
C.../JMDAT1/ and /JMDAT2/.
 
      PARAMETER(MAXV=2000,MAXM=150000,MAXI=1000,MAXO=1000,MAXD=2)
 
      COMMON /JNDAT1/ MSTJN(40),PARJN(40),MSTJM(20),PARJM(20),
     &                OIN(MAXI),OUT(MAXO),MXNDJM
      COMMON /JNINT1/ O(MAXV),A(MAXV),D(MAXV),T(MAXV),DT(MAXV),
     &                W(MAXM),DW(MAXM),NSELF(MAXM),NTSELF(MAXV)
      COMMON /JMINT1/ NDIM,ISW(10),NODES(0:MAXD+1),NBO
      SAVE /JNDAT1/,/JNINT1/,/JMINT1/
 
      DIMENSION INDW(MAXV)
      EQUIVALENCE (NTSELF(1),INDW(1))
 
 
C...Check if JNINIT has been called:
      IF(MSTJN(8).EQ.1) CALL JMERR(10)
 
C...Set parameters:
      CALL JMSEPA
 
C...Set initial values of weights:
      DO 100 IW=1,INDW(NODES(MAXD+1)+1)-1
        IF (MSTJM(2).EQ.0) THEN
          W(IW)=RLU(0)*PARJM(4)
        ELSEIF (MSTJM(2).EQ.1) THEN
          W(IW)=(2.*RLU(0)-1.)*PARJM(4)
        ENDIF
100   CONTINUE
 
C...Normalize weights:
      IF (MSTJM(7).EQ.1) CALL JMNORM
 
C...Write statistics on output file:
 
      IF (MSTJM(6).LT.0) RETURN
 
      CALL JNHEAD
 
      CALL JMSTAT(1)
 
      IF (MSTJM(5).EQ.0) THEN
        WRITE(MSTJM(6),600)
      ELSEIF (MSTJM(5).EQ.1) THEN
        WRITE(MSTJM(6),610)
      ELSEIF (MSTJM(5).EQ.2) THEN
        WRITE(MSTJM(6),660)
      ENDIF
 
      WRITE(MSTJM(6),*)
 
      IF (MSTJM(2).EQ.0) THEN
        WRITE(MSTJM(6),620) PARJM(4)
      ELSEIF (MSTJM(2).EQ.1) THEN
        WRITE(MSTJM(6),630) PARJM(4)
      ENDIF
 
      WRITE(MSTJM(6),*)
 
      IF (MSTJM(7).EQ.0) THEN
        WRITE(MSTJM(6),640)
      ELSEIF (MSTJM(7).EQ.1) THEN
        WRITE(MSTJM(6),650)
      ENDIF
 
      WRITE(MSTJM(6),*)
 
600   FORMAT(26X,'Self-organized Clustering')
610   FORMAT(25X,'Learning Vector Quantization')
620   FORMAT(16X,'Weights set randomly between 0.0 and + ',F6.3)
630   FORMAT(20X,'Weights set randomly between +/- ',F6.3)
640   FORMAT(26X,'Weights are not normalized')
650   FORMAT(28X,'Weights are normalized')
660   FORMAT(24X,'LVQ with neighborhood function')
 
      RETURN
 
C**** END OF JMINIT ****************************************************
      END
C***********************************************************************
 
 
      SUBROUTINE JMINWE(INOD)
C...JetMap subroutine INitial WEight.
 
C...Sets the weight vector for unit INOD equal to the
C...input pattern stored in OIN.
 
      PARAMETER(MAXV=2000,MAXM=150000,MAXI=1000,MAXO=1000,MAXD=2)
 
      COMMON /JNDAT1/ MSTJN(40),PARJN(40),MSTJM(20),PARJM(20),
     &                OIN(MAXI),OUT(MAXO),MXNDJM
      COMMON /JNINT1/ O(MAXV),A(MAXV),D(MAXV),T(MAXV),DT(MAXV),
     &                W(MAXM),DW(MAXM),NSELF(MAXM),NTSELF(MAXV)
      COMMON /JMINT1/ NDIM,ISW(10),NODES(0:MAXD+1),NBO
      SAVE /JNDAT1/,/JNINT1/,/JMINT1/
 
      DIMENSION INDW(MAXV)
      EQUIVALENCE (NTSELF(1),INDW(1))
 
 
      IF (MSTJM(8).NE.1) CALL JMERR(9)
 
      DO 100 K=1,NODES(0)
        IW=INDW(INOD)+K-1
        W(IW)=OIN(K)
100   CONTINUE
 
      RETURN
 
C**** END OF JMINWE ****************************************************
      END
C***********************************************************************
 
 
      SUBROUTINE JMNBHD
C...JetMap subroutine NeighBourHooD.
 
C...Specifies the neighbourhood to update according to MSTJM(9).
 
      PARAMETER(MAXV=2000,MAXM=150000,MAXI=1000,MAXO=1000,MAXD=2)
 
      COMMON /JNDAT1/ MSTJN(40),PARJN(40),MSTJM(20),PARJM(20),
     &                OIN(MAXI),OUT(MAXO),MXNDJM
      COMMON /JNINT1/ O(MAXV),A(MAXV),D(MAXV),T(MAXV),DT(MAXV),
     &                W(MAXM),DW(MAXM),NSELF(MAXM),NTSELF(MAXV)
      COMMON /JMINT1/ NDIM,ISW(10),NODES(0:MAXD+1),NBO
      SAVE /JNDAT1/,/JNINT1/,/JMINT1/
 
      DIMENSION INDW(MAXV),NBHD(MAXO,0:121)
      EQUIVALENCE (NSELF(1),NBHD(1,0)),(NTSELF(1),INDW(1))
 
      IF(MSTJM(8).NE.1) CALL JMERR(14)
 
      IF (ABS(MSTJM(9)).GT.5) THEN
        CALL JMWARN(2)
        MSTJM(9)=SIGN(5,MSTJM(9))
      ENDIF
 
      IF (MSTJM(9).EQ.0) THEN
        DO 200 INOD=1,NODES(MAXD+1)
          NBHD(INOD,1)=INOD
          NBHD(INOD,0)=1
200     CONTINUE
        RETURN
      ENDIF
 
      IF (NDIM.EQ.1) THEN
 
        DO 100 INOD=1,NODES(MAXD+1)
          IF (MSTJM(11).GE.0) THEN
C...Non-periodic boundary:
            ISTRT=MAX(1,INOD-ABS(MSTJM(9)))
            IEND=MIN(MSTJM(11),INOD+ABS(MSTJM(9)))
          ELSE
C...Periodic boundary:
            ISTRT=INOD-ABS(MSTJM(9))
            IEND=ISTRT+2*ABS(MSTJM(9))
          ENDIF
          NBNUM=0
          DO 110 I=ISTRT,IEND
            IND=MOD(I-1,MSTJM(11))+1
            NBNUM=NBNUM+1
            NBHD(INOD,NBNUM)=IND
110       CONTINUE
          NBHD(INOD,0)=NBNUM
100     CONTINUE
 
      ELSEIF (NDIM.EQ.2) THEN
 
        IF (MSTJM(9).GT.0) THEN
C...Square neighbourhood:
          DO 120 INOD=1,NODES(MAXD+1)
            CALL JMINDX(INOD,IC,JC)
            IF (MSTJM(11).GE.0) THEN
C...Non-periodic in dim. 1:
              ISTRT=MAX(1,IC-MSTJM(9))
              IEND=MIN(MSTJM(11),IC+MSTJM(9))
            ELSE
C...Periodic in dim. 1:
              ISTRT=ABS(MSTJM(11))+IC-MSTJM(9)
              ISTRT=MOD(ISTRT-1,ABS(MSTJM(11)))+1
              IEND=ISTRT+2*MSTJM(9)
            ENDIF
            IF (MSTJM(12).GE.0) THEN
C...Non-periodic in dim.2:
              JSTRT=MAX(1,JC-MSTJM(9))
              JEND=MIN(MSTJM(12),JC+MSTJM(9))
            ELSE
C...Periodic in dim. 2:
              JSTRT=ABS(MSTJM(12))+JC-MSTJM(9)
              JSTRT=MOD(JSTRT-1,ABS(MSTJM(12)))+1
              JEND=JSTRT+2*MSTJM(9)
            ENDIF
            NBNUM=0
            DO 130 I=ISTRT,IEND
              DO 140 J=JSTRT,JEND
                IND=MOD(I-1,MSTJM(11))+1
                JND=MOD(J-1,MSTJM(12))+1
                NBNUM=NBNUM+1
                NBNOD=0
                CALL JMINDX(NBNOD,IND,JND)
                NBHD(INOD,NBNUM)=NBNOD
140           CONTINUE
130         CONTINUE
            NBHD(INOD,0)=NBNUM
120       CONTINUE
 
        ELSEIF (MSTJM(9).LT.0) THEN
C...Circular neighbourhood:
          DO 150 INOD=1,NODES(MAXD+1)
            CALL JMINDX(INOD,IC,JC)
            IF (MSTJM(11).GE.0) THEN
C...Non-periodic in dim. 1:
              ISTRT=MAX(1,IC+MSTJM(9))
              IEND=MIN(MSTJM(11),IC-MSTJM(9))
            ELSE
C...Periodic in dim. 1:
              ISTRT=ABS(MSTJM(11))+IC+MSTJM(9)
              ISTRT=MOD(ISTRT-1,ABS(MSTJM(11)))+1
              IEND=ISTRT-2*MSTJM(9)
              IC=ISTRT-MSTJM(9)
            ENDIF
            IF (MSTJM(12).GE.0) THEN
C...Non-periodic in dim.2:
              JSTRT=MAX(1,JC+MSTJM(9))
              JEND=MIN(MSTJM(12),JC-MSTJM(9))
            ELSE
C...Periodic in dim. 2:
              JSTRT=ABS(MSTJM(12))+JC+MSTJM(9)
              JSTRT=MOD(JSTRT-1,ABS(MSTJM(12)))+1
              JEND=JSTRT-2*MSTJM(9)
              JC=JSTRT-MSTJM(9)
            ENDIF
            NBNUM=0
            DO 160 I=ISTRT,IEND
              DO 170 J=JSTRT,JEND
                RDIST=SQRT(FLOAT((IC-I)**2+(JC-J)**2))
                IF (RDIST.LE.FLOAT(-MSTJM(9))) THEN
                  IND=MOD(I-1,ABS(MSTJM(11)))+1
                  JND=MOD(J-1,ABS(MSTJM(12)))+1
                  NBNUM=NBNUM+1
                  NBNOD=0
                  CALL JMINDX(NBNOD,IND,JND)
                  NBHD(INOD,NBNUM)=NBNOD
                ENDIF
170           CONTINUE
160         CONTINUE
            NBHD(INOD,0)=NBNUM
150       CONTINUE
        ENDIF
 
      ENDIF
 
      NBO=MSTJM(9)
 
      RETURN
 
C**** END OF JMNBHD ****************************************************
      END
C***********************************************************************
 
 
      SUBROUTINE JMNORM
C...JetMap subroutine NORMalize weights.
 
C...Normalizes the weights.
 
      PARAMETER(MAXV=2000,MAXM=150000,MAXD=2)
 
      COMMON /JNINT1/ O(MAXV),A(MAXV),D(MAXV),T(MAXV),DT(MAXV),
     &                W(MAXM),DW(MAXM),NSELF(MAXM),NTSELF(MAXV)
      COMMON /JMINT1/ NDIM,ISW(10),NODES(0:MAXD+1),NBO
      SAVE /JNINT1/,/JMINT1/
 
      DIMENSION INDW(MAXV)
      EQUIVALENCE (NTSELF(1),INDW(1))
 
 
      DO 100 INOD=1,NODES(MAXD+1)
        RNORM=0.0
        IWSTRT=INDW(INOD)
        IWEND=INDW(INOD)+NODES(0)-1
        DO 110 IW=IWSTRT,IWEND
          RNORM=RNORM+W(IW)*W(IW)
110     CONTINUE
        RNORM=SQRT(RNORM)
        DO 120 IW=IWSTRT,IWEND
          W(IW)=W(IW)/RNORM
120     CONTINUE
100   CONTINUE
 
      RETURN
 
C**** END OF JMNORM ****************************************************
      END
C***********************************************************************
 
 
      SUBROUTINE JMREAD(NF)
C...JetMap subroutine READ weights and parameters.
 
C...Reads weights and parameters from file NF and initializes the net.
 
      PARAMETER(MAXV=2000,MAXM=150000,MAXI=1000,MAXO=1000,MAXD=2)
 
      COMMON /JNDAT1/ MSTJN(40),PARJN(40),MSTJM(20),PARJM(20),
     &                OIN(MAXI),OUT(MAXO),MXNDJM
      COMMON /JNINT1/ O(MAXV),A(MAXV),D(MAXV),T(MAXV),DT(MAXV),
     &                W(MAXM),DW(MAXM),NSELF(MAXM),NTSELF(MAXV)
      COMMON /JMINT1/ NDIM,ISW(10),NODES(0:MAXD+1),NBO
      SAVE /JNDAT1/,/JNINT1/,/JMINT1/
 
      DIMENSION INDW(MAXV)
      EQUIVALENCE (NTSELF(1),INDW(1))
 
      CHARACTER LINE*1000
 
 
      NDUM=MSTJM(6)
 
      IF(NF.LT.0) THEN
 
C...Unformatted read
       JF=-NF
       READ(JF) IVERS
       IF(IVERS.GE.0) CALL JMERR(12)
       READ(JF) MSTJM,PARJM
 
       CALL JMSEPA
 
       DO 100 IW=1,INDW(NODES(MAXD+1)+1)-1
        READ(JF) W(IW)
100    CONTINUE
 
      ELSE
C...Formatted read
 
        NFSAVE=MSTJM(6)
 
        READ(NF,690)LINE
        IF (LINE(27:28).EQ.'Du') CALL JMERR(12)
        READ(NF,*)
        READ(NF,*)
        READ(NF,710)FVERS
        IVERS=INT(FVERS*10.0+0.001)
 
900     READ(NF,690)LINE
        IF (LINE(1:17).NE.'         I      1') GOTO 900
 
        READ(NF,610)LINE(1:10),(MSTJM(I),I=1,10)
        READ(NF,610)LINE(1:10),(MSTJM(10+I),I=1,10)
        READ(NF,620)LINE(1:10),(PARJM(I),I=1,10)
        READ(NF,620)LINE(1:10),(PARJM(10+I),I=1,10)
        READ(NF,*)
 
        MSTJM(6)=NFSAVE
 
        CALL JMSEPA
 
        IF (NDIM.EQ.1) THEN
          DO 110 INOD=1,NODES(MAXD+1)
            READ(NF,*)
            READ(NF,630)LINE,I
            IW=INDW(INOD)-1
            READ(NF,640)(W(IW+K),K=1,NODES(0))
110       CONTINUE
        ELSEIF(NDIM.EQ.2) THEN
          DO 120 INOD=1,NODES(MAXD+1)
            READ(NF,*)
            READ(NF,650)LINE,I,LINE,J,LINE
            IW=INDW(INOD)-1
            READ(NF,640)(W(IW+K),K=1,NODES(0))
120       CONTINUE
        ENDIF
 
      ENDIF
 
      MSTJM(6)=NDUM
 
C...Write statistics on output file:
      IF (MSTJM(6).LT.0) RETURN
      CALL JNHEAD
      CALL JMSTAT(1)
      WRITE(MSTJM(6),600)
 
600   FORMAT(21X,'Weights read from file')
610   FORMAT(A,10I7)
620   FORMAT(A,10F7.4)
630   FORMAT(A5,I2)
640   FORMAT(10F9.4)
650   FORMAT(A6,I2,A1,I2,A1)
690   FORMAT(A)
700   FORMAT(I2,A)
710   FORMAT(TR63,F2.1)
 
      RETURN
 
C**** END OF JMREAD ****************************************************
      END
C***********************************************************************
 
 
      SUBROUTINE JMSEPA
C...JetMap subroutine SEt PArameters.
 
C...Sets parameters in /JMINT1/.
 
      PARAMETER(MAXV=2000,MAXM=150000,MAXI=1000,MAXO=1000,MAXD=2)
 
      COMMON /JNDAT1/ MSTJN(40),PARJN(40),MSTJM(20),PARJM(20),
     &                OIN(MAXI),OUT(MAXO),MXNDJM
      COMMON /JNINT1/ O(MAXV),A(MAXV),D(MAXV),T(MAXV),DT(MAXV),
     &                W(MAXM),DW(MAXM),NSELF(MAXM),NTSELF(MAXV)
      COMMON /JMINT1/ NDIM,ISW(10),NODES(0:MAXD+1),NBO
      SAVE /JNDAT1/,/JNINT1/,/JMINT1/
 
      DIMENSION INDW(MAXV)
      EQUIVALENCE (NTSELF(1),INDW(1))
 
 
C...Number of dimensions:
      NDIM=MSTJM(1)
      IF (NDIM.GT.MAXD.OR.NDIM.LE.0) CALL JMERR(1)
 
C...Number of nodes:
      IF (MSTJM(10).GT.MAXI) CALL JMERR(2)
      IF (NDIM.EQ.1) THEN
        NODES(MAXD+1)=ABS(MSTJM(11))
      ELSEIF (NDIM.EQ.2) THEN
        NODES(MAXD+1)=ABS(MSTJM(11)*MSTJM(12))
      ENDIF
      IF (NODES(MAXD+1).GT.MAXO) CALL JMERR(3)
      DO 100 IDIM=1,MAXD+1
        NODES(IDIM-1)=ABS(MSTJM(9+IDIM))
100   CONTINUE
 
C *** Set internal switches: ***
C...Response function:
      ISW(1)=MSTJM(3)
C...Error measure:
      ISW(2)=MSTJM(4)
 
C *** Calculate weight pointers: ***
      IF (NODES(MAXD+1)*NODES(0).GT.MAXM) CALL JMERR(4)
      DO 110 INOD=1,NODES(MAXD+1)+1
        INDW(INOD)=(INOD-1)*NODES(0)+1
110   CONTINUE
 
      MSTJM(8)=1
 
C...Initialize neighbourhood:
      CALL JMNBHD
 
      RETURN
 
C**** END OF JMSEPA ****************************************************
      END
C***********************************************************************
 
 
      SUBROUTINE JMSTAT(IS)
 
C...JetMap subroutine output STATistics.
 
C...Statistics chosen by IS is written on file MSTJM(6).
 
      PARAMETER(MAXV=2000,MAXM=150000,MAXI=1000,MAXO=1000,MAXD=2)
 
      COMMON /JNDAT1/ MSTJN(40),PARJN(40),MSTJM(20),PARJM(20),
     &                OIN(MAXI),OUT(MAXO),MXNDJM
      COMMON /JNINT1/ O(MAXV),A(MAXV),D(MAXV),T(MAXV),DT(MAXV),
     &                W(MAXM),DW(MAXM),NSELF(MAXM),NTSELF(MAXV)
      COMMON /JMINT1/ NDIM,ISW(10),NODES(0:MAXD+1),NBO
      SAVE /JNDAT1/,/JNINT1/,/JMINT1/
 
      DIMENSION INDW(MAXV)
      EQUIVALENCE (NTSELF(1),INDW(1))
 
 
      IF(IS.EQ.1) THEN
        WRITE(MSTJM(6),*)
        WRITE(MSTJM(6),600) NDIM
        DO 100 IDIM=1,NDIM
          WRITE(MSTJM(6),610) NODES(IDIM),IDIM
100     CONTINUE
        WRITE(MSTJM(6),620) NODES(0)
        WRITE(MSTJM(6),*)
        WRITE(MSTJM(6),*)
 
      ELSEIF(IS.EQ.2) THEN
        WRITE(MSTJM(6),*)
        WRITE(MSTJM(6),630)
        WRITE(MSTJM(6),*)
        WRITE(MSTJM(6),640)'I',(I,I=1,10)
        WRITE(MSTJM(6),640)'MSTJM I',(MSTJM(I),I=1,10)
        WRITE(MSTJM(6),640)'MSTJM 10+I',(MSTJM(10+I),I=1,10)
        WRITE(MSTJM(6),650)'PARJM I',(PARJM(I),I=1,10)
        WRITE(MSTJM(6),650)'PARJM 10+I',(PARJM(10+I),I=1,10)
        WRITE(MSTJM(6),*)
 
      ELSE
        NWFAC=INDW(NODES(MAXD+1)+1)-1
        WRITE(MSTJM(6),660)NWFAC
      ENDIF
 
600   FORMAT(22X,'Initialized for a ',I1,'-dimensional map with')
610   FORMAT(27X,I3,' nodes in dimension ',I1)
620   FORMAT(27X,I3,' input nodes')
630   FORMAT(18X,'Values of parameters and switches in JETNET')
640   FORMAT(A10,10I7)
650   FORMAT(A10,10F7.4)
660   FORMAT(5X,'Time factor for this map:',I10)
 
      RETURN
 
C**** END OF JMSTAT ****************************************************
      END
C***********************************************************************
 
 
      SUBROUTINE JMTEST
C...JetMap subroutine TEST
 
C...Sets the values of OUT according to given pattern in OIN
C...and current values of weights.
 
      PARAMETER(MAXV=2000,MAXM=150000,MAXI=1000,MAXO=1000,MAXD=2)
 
      COMMON /JNDAT1/ MSTJN(40),PARJN(40),MSTJM(20),PARJM(20),
     &                OIN(MAXI),OUT(MAXO),MXNDJM
      COMMON /JNINT1/ O(MAXV),A(MAXV),D(MAXV),T(MAXV),DT(MAXV),
     &                W(MAXM),DW(MAXM),NSELF(MAXM),NTSELF(MAXV)
      COMMON /JMINT1/ NDIM,ISW(10),NODES(0:MAXD+1),NBO
      SAVE /JNDAT1/,/JNINT1/,/JMINT1/
 
      DIMENSION INDW(MAXV)
      EQUIVALENCE (NTSELF(1),INDW(1))
 
 
      IF (MSTJM(8).NE.1) CALL JMERR(7)
 
      CALL JMFEED
 
      DO 100 INOD=1,NODES(MAXD+1)
        OUT(INOD)=O(INOD)
100   CONTINUE
 
      RETURN
 
C**** END OF JMTEST ****************************************************
      END
C***********************************************************************
 
 
      SUBROUTINE JMTRAL
C...JetMap subroutine TRaining ALgorithm.
 
C...Trains the net according to /JMDAT1/.
 
      PARAMETER(MAXV=2000,MAXM=150000,MAXI=1000,MAXO=1000,MAXD=2)
 
      COMMON /JNDAT1/ MSTJN(40),PARJN(40),MSTJM(20),PARJM(20),
     &                OIN(MAXI),OUT(MAXO),MXNDJM
      COMMON /JNINT1/ O(MAXV),A(MAXV),D(MAXV),T(MAXV),DT(MAXV),
     &                W(MAXM),DW(MAXM),NSELF(MAXM),NTSELF(MAXV)
      COMMON /JMINT1/ NDIM,ISW(10),NODES(0:MAXD+1),NBO
      SAVE /JNDAT1/,/JNINT1/,/JMINT1/
 
      DIMENSION INDW(MAXV),NBHD(MAXO,0:121)
      EQUIVALENCE (NSELF(1),NBHD(1,0)),(NTSELF(1),INDW(1))
 
 
      IF (MSTJM(8).NE.1) CALL JMERR(6)
 
C...Update neighbourhood:
      IF (MSTJM(9).NE.NBO) CALL JMNBHD
 
      CALL JMFEED
      IF (MXNDJM.EQ.0) RETURN
 
      IF (MSTJM(5).EQ.0) THEN
C...Normal updating
        DO 100 INBH=1,NBHD(MXNDJM,0)
          IWSTRT=INDW(NBHD(MXNDJM,INBH))
          IWEND=INDW(NBHD(MXNDJM,INBH)+1)-1
          DO 110 IW=IWSTRT,IWEND
            W(IW)=W(IW)+PARJM(1)*DW(IW)
110       CONTINUE
100     CONTINUE
 
      ELSEIF (MSTJM(5).EQ.1) THEN
C...Learning Vector Quantization
        IWSTRT=INDW(MXNDJM)
        IWEND=INDW(MXNDJM+1)-1
        IF (OUT(MXNDJM).LT.0.5) THEN
C...Wrong answer:
          DO 200 IW=IWSTRT,IWEND
            W(IW)=W(IW)-PARJM(1)*DW(IW)
200       CONTINUE
        ELSE
C...Correct answer:
          DO 210 IW=IWSTRT,IWEND
            W(IW)=W(IW)+PARJM(1)*DW(IW)
210       CONTINUE
        ENDIF
 
      ELSEIF (MSTJM(5).EQ.2) THEN
C...LVQ with neighborhood function:
        IWSTRT=INDW(MXNDJM)
        IWEND=INDW(MXNDJM+1)-1
        DO 220 IW=IWSTRT,IWEND
          W(IW)=W(IW)+PARJM(1)*DW(IW)*OUT(MXNDJM)
220     CONTINUE
 
      ENDIF
 
      DO 300 INOD=1,NODES(MAXD+1)
        OUT(INOD)=O(INOD)
300   CONTINUE
 
C...If normalize:
      IF (MSTJM(7).EQ.1) CALL JMNORM
 
      RETURN
 
C**** END OF JMTRAL ****************************************************
      END
C***********************************************************************
 
 
      SUBROUTINE JMWARN(IWARN)
C...JetMap subroutine WARNing.
 
C...Writes out a warning on file MSTJM(6).
 
      PARAMETER(MAXI=1000,MAXO=1000)
 
      COMMON /JNDAT1/ MSTJN(40),PARJN(40),MSTJM(20),PARJM(20),
     &                OIN(MAXI),OUT(MAXO),MXNDJM
      SAVE /JNDAT1/
 
 
      MSTJN(34)=MSTJN(34)+1
      MSTJN(33)=IWARN
      IF(MSTJN(34).LT.MSTJN(32)) THEN
 
       WRITE(MSTJM(6),600)IWARN
 
       IF (IWARN.EQ.1) THEN
         WRITE(MSTJM(6),610)
       ELSEIF (IWARN.EQ.2) THEN
         WRITE(MSTJM(6),620)MSTJM(9)
         WRITE(MSTJM(6),625)
         WRITE(MSTJM(6),630)
       ENDIF
 
      ELSEIF(MSTJN(31).GE.1) THEN
 
       CALL JMERR(13)
 
      ENDIF
 
600   FORMAT(' *** JETMAP WARNING:',I2,' ***')
610   FORMAT(' No response in net for presented input')
620   FORMAT(' Illegal value of neighbourhood size (',I2,')')
625   FORMAT(' absolute value of MSTJM(9) must be within [0,5]')
630   FORMAT(' MSTJM(9) set to limit value 5')
 
      RETURN
 
C**** END OF JMWARN ****************************************************
      END
C***********************************************************************
 
 
      SUBROUTINE JNTDEC
C...JetNet subroutine Test-DECk
 
C...Runs a test-program using data from two overlapping Gaussian
C...distributions in the input space. The test-program uses the
C...back-propagation training algorithm.
 
      PARAMETER(MAXI=1000,MAXO=1000)
      PARAMETER(MXPT=10000,NTRAIN=5000,NTEST=5000,ND=3)
 
      COMMON /JNDAT1/ MSTJN(40),PARJN(40),MSTJM(20),PARJM(20),
     &                OIN(MAXI),OUT(MAXO),MXNDJM
      COMMON /JNTDC1/ TDIN(MXPT,10),TDOUT(MXPT)
      SAVE /JNDAT1/,/JNTDC1/
 
 
      WRITE(MSTJN(6),600)
 
 
      WRITE(MSTJN(6),610)ND
      WRITE(MSTJN(6),620)
 
C...Set network architecture: MSTJN(1)-layered network with
C...MSTJN(11) hidden nodes, MSTJN(12) output nodes and
C...MSTJN(10) inputs.
      MSTJN(1)=3
      MSTJN(10)=ND
      MSTJN(11)=6
      MSTJN(12)=1
 
C...Set sigmoid function:
      MSTJN(3)=2
 
C...Initial width of weights:
      PARJN(4)=0.01
 
C...Initialize network:
      CALL JNINIT
 
C...Get patterns:
      CALL JNTDPT
 
C...Set training parameters:
C...Learning rate:
      PARJN(1)=0.01
C...Momentum:
      PARJN(2)=0.5
 
C...All other parameters and switches keep their default values.
 
 
      WRITE(MSTJN(6),*)
      WRITE(MSTJN(6),630)
      NEPOCH=100
C...Parameter to change learning rate during execution:
      DETA=PARJN(1)*0.1/FLOAT(NEPOCH)
 
C...Main loop over epochs:
      DO 100 IEPOCH=1,NEPOCH
 
 
C...Training loop:
        NRIGHT=0
        DO 110 IP=1,NTRAIN
 
C...Select pattern randomly to avoid cyclic behaviour:
          IPAT=INT(RLU(0)*NTRAIN)+1
C...Put pattern into OIN:
          DO 120 I=1,MSTJN(10)
            OIN(I)=TDIN(IPAT,I)
120       CONTINUE
C...Put target output value into OUT:
          OUT(1)=TDOUT(IPAT)
 
C...Invoke training algorithm:
          CALL JNTRAL
 
C...Calculate performance on training set:
          IF (ABS(OUT(1)-TDOUT(IPAT)).LT.0.5) NRIGHT=NRIGHT+1
 
110     CONTINUE
        TRAIN=FLOAT(NRIGHT)/FLOAT(NTRAIN)
 
 
C...Testing loop:
        NRIGHT=0
        DO 130 IP=1,NTEST
          IPAT=NTRAIN+IP
C...Put pattern into OIN:
          DO 140 I=1,MSTJN(10)
            OIN(I)=TDIN(IPAT,I)
140       CONTINUE
 
C...Get network output:
          CALL JNTEST
 
C...Calculate performance on test set (=generalization):
          IF (ABS(OUT(1)-TDOUT(IPAT)).LT.0.5) NRIGHT=NRIGHT+1
130     CONTINUE
        TEST=FLOAT(NRIGHT)/FLOAT(NTEST)
 
 
C...Display performance:
        WRITE(MSTJN(6),640)IEPOCH,TRAIN,TEST
 
C...Update learning rate:
        PARJN(1)=PARJN(1)-DETA
 
100   CONTINUE
 
      WRITE(MSTJN(6),*)
      WRITE(MSTJN(6),650)
      WRITE(MSTJN(6),660)
 
      WRITE(MSTJN(6),*)
      IF (ABS(TEST-0.75).LT.0.02.OR.ABS(TRAIN-0.75).LT.0.02) THEN
        WRITE(MSTJN(6),670)
      ENDIF
 
 
600   FORMAT(31X,'JETNET Test-Deck')
610   FORMAT(15X,'Two overlapping Gaussian distributions in ',
     &I2,' dim.')
620   FORMAT(24X,'Using back-propagation network')
630   FORMAT('   Epoch   /  Training  /  General. ')
640   FORMAT(I8,2X,2(' /',F9.2,2X))
650   FORMAT(' If nothing is changed in JNTDEC,')
660   FORMAT(' the performance should be about 0.74-0.76.')
670   FORMAT(' Congratulations, it is working.')
 
      RETURN
 
C**** END OF JNTDEC ****************************************************
      END
C***********************************************************************
 
 
      SUBROUTINE JNTDPT
C...JetNet subroutine Test-Deck PaTterns
 
C...Produces patterns used in JNTDEC for training and testing.
C...The patterns are picked from two overlapping Gaussian distributions
C...in MSTJM(10)-dimensional space. The two distributions have the same
C...center points but different widths -> The network has to cut out a
C...closed volume in input space, which can be considered a 'hard'
C...problem.
 
      PARAMETER(MAXI=1000,MAXO=1000)
      PARAMETER(MXPT=10000,NTRAIN=5000,NTEST=5000,PI=3.1415927)
 
      COMMON /JNDAT1/ MSTJN(40),PARJN(40),MSTJM(20),PARJM(20),
     &                OIN(MAXI),OUT(MAXO),MXNDJM
      COMMON /JNTDC1/ TDIN(MXPT,10),TDOUT(MXPT)
      SAVE /JNDAT1/,/JNTDC1/
 
      DIMENSION VEC(10)
 
 
C...The center-point is, for simplicity, the origin.
 
C...Widths of the two distributions: (WID1>WID2)
      WID1=1.0
      WID2=0.3
 
C...Loop over the number of patterns:
      DO 100 IPAT=1,MXPT
 
C...Select random direction from center point:
        VNORM=0.0
        DO 110 I=1,MSTJN(10)
          IF (RLU(0).LT.0.5) THEN
            VEC(I)=RLU(0)
          ELSE
            VEC(I)=-RLU(0)
          ENDIF
          VNORM=VNORM+VEC(I)*VEC(I)
110     CONTINUE
        VNORM=SQRT(VNORM)
 
        GWID=GAUSJN(0)
 
        IF (RLU(0).LT.0.5) THEN
C...Class one:
          DO 120 I=1,MSTJN(10)
            TDIN(IPAT,I)=WID1*GWID*VEC(I)/VNORM
120       CONTINUE
          TDOUT(IPAT)=1.0
        ELSE
C...Class two:
          DO 130 I=1,MSTJN(10)
            TDIN(IPAT,I)=WID2*GWID*VEC(I)/VNORM
130       CONTINUE
          TDOUT(IPAT)=0.0
        ENDIF
 
100   CONTINUE
 
      RETURN
 
C**** END OF JNTDPT ****************************************************
      END
C***********************************************************************
 
 
      BLOCK DATA JNDATA
 
C...JetNet block DATA
 
C...Initial values for parameters and switches for JETNET.
 
      PARAMETER(MAXV=2000,MAXM=150000,MAXI=1000,MAXO=1000,MAXD=2)
 
      COMMON /JNDAT1/ MSTJN(40),PARJN(40),MSTJM(20),PARJM(20),
     &                OIN(MAXI),OUT(MAXO),MXNDJM
      COMMON /JNDAT2/ TINV(10),IGFN(10),ETAL(10),WIDL(10),SATM(10)
      COMMON /JNINT1/ O(MAXV),A(MAXV),D(MAXV),T(MAXV),DT(MAXV),
     &                W(MAXM),DW(MAXM),NSELF(MAXM),NTSELF(MAXV)
      COMMON /JNINT2/ M(0:10),MV0(10),MM0(10),NG(10),NL,IPOTT,
     &                ER1,ER2,SM(10),ICPON
      COMMON /JMINT1/ NDIM,ISW(10),NODES(0:MAXD+1),NBO
      COMMON /JNGAUS/ ISET,GASDEV
 
 
C...Brief explanation of parameters and switches :
C...
C.../JNDAT1/:
C...Feed-forward net:
C...MSTJN(1) (D=3)      number of layers in net
C...MSTJN(2) (D=10)     number of patterns per update in JNTRAL
C...MSTJN(3) (D=1)      overall transfer function used in net
C...        1 -> g(x)=1/(1+exp(-2x))
C...        2 -> g(x)=tanh(x)
C...        3 -> g(x)=exp(x) (only used internally for Potts-nodes)
C...        4 -> g(x)=x
C...        5 -> g(x)=1/(1+exp(-2x)) (only used internally for
C...             entropy error)
C...MSTJN(4) (D=0)      error measure
C...        0 -> summed square error
C...        1 -> entropy error
C...      >=2 -> Kullback measure, using Potts nodes of dimension
C...             MSTJN(4)
C...MSTJN(5) (D=0)      updating procedure
C...        0 -> normal updating
C...        1 -> Manhattan updating
C...        2 -> Langevin updating
C...MSTJN(6) (D=6)      file number for output statistics
C...MSTJN(7) (I)        number of calls to JNTRAL
C...MSTJN(8) (I)        initialization done
C...MSTJN(9) (D=100)    number of updates per epoch
C...MSTJN(10+I)         number of nodes in layer I (I=0 => input layer)
C...MSTJN(10) (D=16)
C...MSTJN(11) (D=8)
C...MSTJN(12) (D=1)
C...MSTJN(13-20) (D=0)
C...MSTJN(21) (D=0)     pruning (>0 -> on)
C...MSTJN(22) (D=0)     saturation measure (<>0 -> on)
C...                    <0 -> update temperature to give measure ~0.5
C...MSTJN(23,24) (D=0)  geometry of input nodes for receptive fields
C...MSTJN(25,26) (D=0)  geometry of receptive fields
C...MSTJN(27)    (D=1)  number of hidden nodes per receptive field
C...MSTJN(28-30) (D=0)  precision in bits (0 -> machine precision) for
C...                    sigmoid functions (28), thresholds (29) and
C...                    weights (30)
C...MSTJN(31)    (D=1)  Warning procedure
C...        0 -> No action is taken after a warning
C...        1 -> The execution is stopped after the program
C...             has experienced MSTJN(32) warnings
C...             in any case only MSTJN(32) warning messages are printed
C...             out.
C...MSTJN(32) (D=10)    Maximum number of warning messages to be
C...                    printed. As described above.
C...MSTJN(33) (I)       code for latest warning issued by the program.
C...MSTJN(34) (I)       Number of warnings issued by the program so far.
C...MSTJN(35-40)        not used
C...
C...
C...PARJN(1) (D=0.001)  learning parameter eta
C...PARJN(2) (D=0.5)    momentum term alfa
C...PARJN(3) (D=1.0)    overall inverse temperature beta
C...PARJN(4) (D=0.1)    width of initial weights
C...PARJN(5) (D=0.0)    forgetting parameter epsilon
C...PARJN(6) (D=0.0)    noise width in Langevin equation
C...PARJN(7) (R)        last error per node
C...PARJN(8) (R)        mean error in last update
C...PARJN(9) (R)        mean error last epoch (equal to MSTJN(9) updates)
C...PARJN(10)(R)        weighted mean average used in pruning
C...PARJN(11) (D=1.0)   decrease in eta (scale factor per epoch)
C...!PARJN(12) (D=1.0)  decrease in 1-alpha (scale factor per epoch)
C...PARJN(13) (D=1.0)   decrease in temperature (scale factor per epoch)
C...PARJN(14) (D=0.0)   pruning parameter lambda
C...PARJN(15) (D=1.E-6) change in lambda
C...PARJN(16) (D=0.9)   parameter gamma used for calculation of PARJN(10)
C...PARJN(17) (D=0.9)   pruning "cut-off"
C...PARJN(18) (D=1.0)   scale parameter W(0), used in pruning
C...PARJN(19) (D=0.0)   target error when pruning
C...PARJN(20-40)        not used
C...
C...
C...Self-organizing net:
C...MSTJM(1)   (D=1)    number of dimensions in net
C...MSTJM(2)   (D=0)    symmetry of initial weights
C...        0 -> [0,+width]
C...        1 -> [-width,+width]
C...MSTJM(3)   (D=2)    response function
C...        1 -> g(x)=0.5*(1.0+tanh(x) : for normalized data
C...        2 -> g(x)=exp(-x)          : for unnormalized data
C...MSTJM(4)   (D=1)    error measure
C...        1 -> summed square error
C...MSTJM(5)   (D=0)    updating procedure
C...        0 -> unsupervized clustering & topological ordering
C...        1 -> Learning Vector Quantization (LVQ 1)
C...        2 -> as 1, but with neighborhood function.
C...MSTJM(6)   (D=6)     output file number
C...MSTJM(7)   (D=0)     normalize weights or not
C...        0 -> unnormalized
C...        1 -> normalized
C...MSTJM(8) (I)         initialization done
C...MSTJM(9)   (D=0)     neighbourhood size
C...        0< -> square neighbourhood
C...        <0 -> circular neighbourhood
C...MSTJM(10)  (D=8)     number of input nodes
C...MSTJM(11)  (D=10)    number of nodes in dimension 1.
C...        <0 -> periodic boundary
C...MSTJM(12)  (D=1)     number of nodes in dimension 2.
C...        <0 -> periodic boundary
C...MSTJM(13-20)         not used
C...
C...
C...PARJM(1)   (D=0.001) learning parameter eta
C...PARJM(2)   (D=0.0)   not used
C...PARJM(3)   (D=0.01)  overall inverse temperature beta
C...PARJM(4)   (D=0.5)   initial width of weights
C...PARJM(5-20)          not used
C...
C...
C.../JNDAT2/:
C...TINV(I) (D=0.0)     inverse temperature of layer I (if 0 use PARJN(3))
C...
C...IGFN(I) (D=0)       sigmoid function for layer I (if 0 use MSTJN(3))
C...
C...ETAL(I) (D=0)       learning parameter in layer I (if 0 use PARJN(1))
C...
C...WIDL(I) (D=0)       initial width in layer I (if 0 use PARJN(4))
C...
C...SATM(I) (R)         saturation measure "S" for layer I.
C...      MSTJN(3)=1 -> S = sum[(1.-2.*O(J))**2]
C...      MSTJN(3)=2 -> S = sum[O(J)**2]
C...
C...End of description
 
 
      DATA MSTJN/3,10,1,0,0,6,0,0,100,16,8,1,14*0,1,3*0,1,10,8*0/
      DATA PARJN/0.001,0.5,1.0,0.1,6*0.0,
     &           3*1.0,0.0,0.000001,0.9,0.9,1.0,22*0.0/
      DATA MSTJM/1,0,2,1,0,6,0,0,0,10,10,1,8*0/
      DATA PARJM/0.001,0.0,0.01,0.5,16*0.0/
      DATA TINV/10*0.0/
      DATA IGFN/10*0/
      DATA ETAL/10*0.0/
      DATA WIDL/10*0.0/
      DATA SATM/10*0.0/
      DATA ER1/0.0/
      DATA ER2/0.0/
      DATA SM/10*0.0/
      DATA NBO/10/
      DATA ICPON/0/
      DATA ISET/0/
      DATA GASDEV/0.0/
 
C**** END OF JNDATA ****************************************************
      END
